@@ -1,6 +1,6 @@
 /* blocked.c - generic support for blocking operations like BLPOP & WAIT.
  *
- * Copyright (c) 2009-2012, Salvatore Sanfilippo <antirez at gmail dot com>
+ * Copyright (c) 2009-2012, Redis Ltd.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -64,6 +64,7 @@
 #include "slowlog.h"
 #include "latency.h"
 #include "monotonic.h"
+#include "cluster_slot_stats.h"
 
 /* forward declarations */
 static void unblockClientWaitingData(client *c);
@@ -75,10 +76,13 @@ static void releaseBlockedEntry(client *c, dictEntry *de, int remove_key);
 void initClientBlockingState(client *c) {
     c->bstate.btype = BLOCKED_NONE;
     c->bstate.timeout = 0;
+    c->bstate.unblock_on_nokey = 0;
     c->bstate.keys = dictCreate(&objectKeyHeapPointerValueDictType);
     c->bstate.numreplicas = 0;
+    c->bstate.numlocal = 0;
     c->bstate.reploffset = 0;
-    c->bstate.unblock_on_nokey = 0;
+    c->bstate.generic_blocked_list_node = NULL;
+    c->bstate.module_blocked_handle = NULL;
     c->bstate.async_rm_call_handle = NULL;
 }
 
@@ -86,14 +90,13 @@ void initClientBlockingState(client *c) {
  * flag is set client query buffer is not longer processed, but accumulated,
  * and will be processed when the client is unblocked. */
 void blockClient(client *c, int btype) {
-    /* Master client should never be blocked unless pause or module */
-    serverAssert(!(c->flags & CLIENT_MASTER &&
-                   btype != BLOCKED_MODULE &&
-                   btype != BLOCKED_POSTPONE));
+    /* Primary client should never be blocked unless pause or module */
+    serverAssert(!(c->flag.primary && btype != BLOCKED_MODULE && btype != BLOCKED_POSTPONE));
 
-    c->flags |= CLIENT_BLOCKED;
+    c->flag.blocked = 1;
     c->bstate.btype = btype;
-    if (!(c->flags & CLIENT_MODULE)) server.blocked_clients++; /* We count blocked client stats on regular clients and not on module clients */
+    if (!c->flag.module)
+        server.blocked_clients++; /* We count blocked client stats on regular clients and not on module clients */
     server.blocked_clients_by_type[btype]++;
     addClientToTimeoutTable(c);
 }
@@ -102,21 +105,22 @@ void blockClient(client *c, int btype) {
  * he will attempt to reprocess the command which will update the statistics.
  * However in case the client was timed out or in case of module blocked client is being unblocked
  * the command will not be reprocessed and we need to make stats update.
- * This function will make updates to the commandstats, slowlog and monitors.*/
-void updateStatsOnUnblock(client *c, long blocked_us, long reply_us, int had_errors){
+ * This function will make updates to the commandstats, slot-stats, slowlog and monitors.*/
+void updateStatsOnUnblock(client *c, long blocked_us, long reply_us, int had_errors) {
     const ustime_t total_cmd_duration = c->duration + blocked_us + reply_us;
     c->lastcmd->microseconds += total_cmd_duration;
+    clusterSlotStatsAddCpuDuration(c, total_cmd_duration);
     c->lastcmd->calls++;
+    c->commands_processed++;
     server.stat_numcommands++;
-    if (had_errors)
-        c->lastcmd->failed_calls++;
+    if (had_errors) c->lastcmd->failed_calls++;
     if (server.latency_tracking_enabled)
-        updateCommandLatencyHistogram(&(c->lastcmd->latency_histogram), total_cmd_duration*1000);
+        updateCommandLatencyHistogram(&(c->lastcmd->latency_histogram), total_cmd_duration * 1000);
     /* Log the command into the Slow log if needed. */
     slowlogPushCurrentCommand(c, c->lastcmd, total_cmd_duration);
     c->duration = 0;
     /* Log the reply duration event. */
-    latencyAddSampleIfNeeded("command-unblocking",reply_us/1000);
+    latencyAddSampleIfNeeded("command-unblocking", reply_us / 1000);
 }
 
 /* This function is called in the beforeSleep() function of the event loop
@@ -130,11 +134,11 @@ void processUnblockedClients(void) {
         ln = listFirst(server.unblocked_clients);
         serverAssert(ln != NULL);
         c = ln->value;
-        listDelNode(server.unblocked_clients,ln);
-        c->flags &= ~CLIENT_UNBLOCKED;
+        listDelNode(server.unblocked_clients, ln);
+        c->flag.unblocked = 0;
 
-        if (c->flags & CLIENT_MODULE) {
-            if (!(c->flags & CLIENT_BLOCKED)) {
+        if (c->flag.module) {
+            if (!c->flag.blocked) {
                 moduleCallCommandUnblockedHandler(c);
             }
             continue;
@@ -144,10 +148,10 @@ void processUnblockedClients(void) {
          * is blocked again. Actually processInputBuffer() checks that the
          * client is not blocked before to proceed, but things may change and
          * the code is conceptually more correct this way. */
-        if (!(c->flags & CLIENT_BLOCKED)) {
+        if (!c->flag.blocked) {
             /* If we have a queued command, execute it now. */
             if (processPendingCommandAndInputBuffer(c) == C_ERR) {
-                c = NULL;
+                continue;
             }
         }
         beforeNextClient(c);
@@ -173,27 +177,26 @@ void processUnblockedClients(void) {
 void queueClientForReprocessing(client *c) {
     /* The client may already be into the unblocked list because of a previous
      * blocking operation, don't add back it into the list multiple times. */
-    if (!(c->flags & CLIENT_UNBLOCKED)) {
-        c->flags |= CLIENT_UNBLOCKED;
-        listAddNodeTail(server.unblocked_clients,c);
+    if (!c->flag.unblocked) {
+        c->flag.unblocked = 1;
+        listAddNodeTail(server.unblocked_clients, c);
     }
 }
 
 /* Unblock a client calling the right function depending on the kind
  * of operation the client is blocking for. */
 void unblockClient(client *c, int queue_for_reprocessing) {
-    if (c->bstate.btype == BLOCKED_LIST ||
-        c->bstate.btype == BLOCKED_ZSET ||
-        c->bstate.btype == BLOCKED_STREAM) {
+    if (c->bstate.btype == BLOCKED_LIST || c->bstate.btype == BLOCKED_ZSET || c->bstate.btype == BLOCKED_STREAM) {
         unblockClientWaitingData(c);
-    } else if (c->bstate.btype == BLOCKED_WAIT || c->bstate.btype == BLOCKED_WAITAOF) {
+    } else if (c->bstate.btype == BLOCKED_WAIT) {
         unblockClientWaitingReplicas(c);
     } else if (c->bstate.btype == BLOCKED_MODULE) {
         if (moduleClientIsBlockedOnKeys(c)) unblockClientWaitingData(c);
         unblockClientFromModule(c);
     } else if (c->bstate.btype == BLOCKED_POSTPONE) {
-        listDelNode(server.postponed_clients,c->postponed_list_node);
-        c->postponed_list_node = NULL;
+        serverAssert(c->bstate.postponed_list_node);
+        listDelNode(server.postponed_clients, c->bstate.postponed_list_node);
+        c->bstate.postponed_list_node = NULL;
     } else if (c->bstate.btype == BLOCKED_SHUTDOWN) {
         /* No special cleanup. */
     } else {
@@ -202,7 +205,7 @@ void unblockClient(client *c, int queue_for_reprocessing) {
 
     /* Reset the client for a new query, unless the client has pending command to process
      * or in case a shutdown operation was canceled and we are still in the processCommand sequence  */
-    if (!(c->flags & CLIENT_PENDING_COMMAND) && c->bstate.btype != BLOCKED_SHUTDOWN) {
+    if (!c->flag.pending_command && c->bstate.btype != BLOCKED_SHUTDOWN) {
         freeClientOriginalArgv(c);
         /* Clients that are not blocked on keys are not reprocessed so we must
          * call reqresAppendResponse here (for clients blocked on key,
@@ -212,50 +215,39 @@ void unblockClient(client *c, int queue_for_reprocessing) {
         resetClient(c);
     }
 
+    /* We count blocked client stats on regular clients and not on module clients */
+    if (!c->flag.module) server.blocked_clients--;
+    server.blocked_clients_by_type[c->bstate.btype]--;
     /* Clear the flags, and put the client in the unblocked list so that
      * we'll process new commands in its query buffer ASAP. */
-    if (!(c->flags & CLIENT_MODULE)) server.blocked_clients--; /* We count blocked client stats on regular clients and not on module clients */
-    server.blocked_clients_by_type[c->bstate.btype]--;
-    c->flags &= ~CLIENT_BLOCKED;
+    c->flag.blocked = 0;
     c->bstate.btype = BLOCKED_NONE;
     c->bstate.unblock_on_nokey = 0;
     removeClientFromTimeoutTable(c);
     if (queue_for_reprocessing) queueClientForReprocessing(c);
 }
 
-/* Check if the specified client can be safely timed out using
- * unblockClientOnTimeout(). */
-int blockedClientMayTimeout(client *c) {
-    if (c->bstate.btype == BLOCKED_MODULE) {
-        return moduleBlockedClientMayTimeout(c);
-    }
-
-    if (c->bstate.btype == BLOCKED_LIST ||
-        c->bstate.btype == BLOCKED_ZSET ||
-        c->bstate.btype == BLOCKED_STREAM ||
-        c->bstate.btype == BLOCKED_WAIT) {
-        return 1;
-    }
-    return 0;
-}
-
 /* This function gets called when a blocked client timed out in order to
  * send it a reply of some kind. After this function is called,
  * unblockClient() will be called with the same client as argument. */
 void replyToBlockedClientTimedOut(client *c) {
-    if (c->bstate.btype == BLOCKED_LIST ||
-        c->bstate.btype == BLOCKED_ZSET ||
-        c->bstate.btype == BLOCKED_STREAM) {
+    if (c->bstate.btype == BLOCKED_LIST || c->bstate.btype == BLOCKED_ZSET || c->bstate.btype == BLOCKED_STREAM) {
         addReplyNullArray(c);
         updateStatsOnUnblock(c, 0, 0, 0);
     } else if (c->bstate.btype == BLOCKED_WAIT) {
-        addReplyLongLong(c,replicationCountAcksByOffset(c->bstate.reploffset));
-    } else if (c->bstate.btype == BLOCKED_WAITAOF) {
-        addReplyArrayLen(c,2);
-        addReplyLongLong(c,server.fsynced_reploff >= c->bstate.reploffset);
-        addReplyLongLong(c,replicationCountAOFAcksByOffset(c->bstate.reploffset));
+        if (c->cmd->proc == waitCommand) {
+            addReplyLongLong(c, replicationCountAcksByOffset(c->bstate.reploffset));
+        } else if (c->cmd->proc == waitaofCommand) {
+            addReplyArrayLen(c, 2);
+            addReplyLongLong(c, server.fsynced_reploff >= c->bstate.reploffset);
+            addReplyLongLong(c, replicationCountAOFAcksByOffset(c->bstate.reploffset));
+        } else if (c->cmd->proc == clusterCommand) {
+            addReplyErrorObject(c, shared.noreplicaserr);
+        } else {
+            serverPanic("Unknown wait command %s in replyToBlockedClientTimedOut().", c->cmd->declared_name);
+        }
     } else if (c->bstate.btype == BLOCKED_MODULE) {
-        moduleBlockedClientTimedOut(c);
+        moduleBlockedClientTimedOut(c, 0);
     } else {
         serverPanic("Unknown btype in replyToBlockedClientTimedOut().");
     }
@@ -268,9 +260,9 @@ void replyToClientsBlockedOnShutdown(void) {
     listNode *ln;
     listIter li;
     listRewind(server.clients, &li);
-    while((ln = listNext(&li))) {
+    while ((ln = listNext(&li))) {
         client *c = listNodeValue(ln);
-        if (c->flags & CLIENT_BLOCKED && c->bstate.btype == BLOCKED_SHUTDOWN) {
+        if (c->flag.blocked && c->bstate.btype == BLOCKED_SHUTDOWN) {
             addReplyError(c, "Errors trying to SHUTDOWN. Check logs.");
             unblockClient(c, 1);
         }
@@ -279,8 +271,8 @@ void replyToClientsBlockedOnShutdown(void) {
 
 /* Mass-unblock clients because something changed in the instance that makes
  * blocking no longer safe. For example clients blocked in list operations
- * in an instance which turns from master to slave is unsafe, so this function
- * is called when a master turns into a slave.
+ * in an instance which turns from primary to replica is unsafe, so this function
+ * is called when a primary turns into a replica.
  *
  * The semantics is to send an -UNBLOCKED error to the client, disconnecting
  * it at the same time. */
@@ -288,27 +280,25 @@ void disconnectAllBlockedClients(void) {
     listNode *ln;
     listIter li;
 
-    listRewind(server.clients,&li);
-    while((ln = listNext(&li))) {
+    listRewind(server.clients, &li);
+    while ((ln = listNext(&li))) {
         client *c = listNodeValue(ln);
 
-        if (c->flags & CLIENT_BLOCKED) {
+        if (c->flag.blocked) {
             /* POSTPONEd clients are an exception, when they'll be unblocked, the
              * command processing will start from scratch, and the command will
              * be either executed or rejected. (unlike LIST blocked clients for
              * which the command is already in progress in a way. */
-            if (c->bstate.btype == BLOCKED_POSTPONE)
-                continue;
+            if (c->bstate.btype == BLOCKED_POSTPONE) continue;
 
-            unblockClientOnError(c,
-                "-UNBLOCKED force unblock from blocking operation, "
-                "instance state changed (master -> replica?)");
-            c->flags |= CLIENT_CLOSE_AFTER_REPLY;
+            unblockClientOnError(c, "-UNBLOCKED force unblock from blocking operation, "
+                                    "instance state changed (master -> replica?)");
+            c->flag.close_after_reply = 1;
         }
     }
 }
 
-/* This function should be called by Redis every time a single command,
+/* This function should be called by the server every time a single command,
  * a MULTI/EXEC block, or a Lua script, terminated its execution after
  * being called by a client. It handles serving clients blocked in all scenarios
  * where a specific key access requires to block until that key is available.
@@ -326,15 +316,13 @@ void disconnectAllBlockedClients(void) {
  * do client side, indeed!). Because mismatching clients (blocking for
  * a different type compared to the current key type) are moved in the
  * other side of the linked list. However as long as the key starts to
- * be used only for a single type, like virtually any Redis application will
+ * be used only for a single type, like virtually any application will
  * do, the function is already fair. */
 void handleClientsBlockedOnKeys(void) {
-
     /* In case we are already in the process of unblocking clients we should
      * not make a recursive call, in order to prevent breaking fairness. */
     static int in_handling_blocked_clients = 0;
-    if (in_handling_blocked_clients)
-        return;
+    if (in_handling_blocked_clients) return;
     in_handling_blocked_clients = 1;
 
     /* This function is called only when also_propagate is in its basic state
@@ -344,7 +332,7 @@ void handleClientsBlockedOnKeys(void) {
     /* If a command being unblocked causes another command to get unblocked,
      * like a BLMOVE would do, then the new unblocked command will get processed
      * right away rather than wait for later. */
-    while(listLength(server.ready_keys) != 0) {
+    while (listLength(server.ready_keys) != 0) {
         list *l;
 
         /* Point server.ready_keys to a fresh list and save the current one
@@ -354,20 +342,20 @@ void handleClientsBlockedOnKeys(void) {
         l = server.ready_keys;
         server.ready_keys = listCreate();
 
-        while(listLength(l) != 0) {
+        while (listLength(l) != 0) {
             listNode *ln = listFirst(l);
             readyList *rl = ln->value;
 
             /* First of all remove this key from db->ready_keys so that
              * we can safely call signalKeyAsReady() against this key. */
-            dictDelete(rl->db->ready_keys,rl->key);
+            dictDelete(rl->db->ready_keys, rl->key);
 
             handleClientsBlockedOnKey(rl);
 
             /* Free this item. */
             decrRefCount(rl->key);
             zfree(rl);
-            listDelNode(l,ln);
+            listDelNode(l, ln);
         }
         listRelease(l); /* We have the new list on place at this point. */
     }
@@ -386,7 +374,7 @@ void blockForKeys(client *c, int btype, robj **keys, int numkeys, mstime_t timeo
     list *l;
     int j;
 
-    if (!(c->flags & CLIENT_REPROCESSING_COMMAND)) {
+    if (!c->flag.reprocessing_command) {
         /* If the client is re-processing the command, we do not set the timeout
          * because we need to retain the client's original timeout. */
         c->bstate.timeout = timeout;
@@ -394,13 +382,13 @@ void blockForKeys(client *c, int btype, robj **keys, int numkeys, mstime_t timeo
 
     for (j = 0; j < numkeys; j++) {
         /* If the key already exists in the dictionary ignore it. */
-        if (!(client_blocked_entry = dictAddRaw(c->bstate.keys,keys[j],NULL))) {
+        if (!(client_blocked_entry = dictAddRaw(c->bstate.keys, keys[j], NULL))) {
             continue;
         }
         incrRefCount(keys[j]);
 
         /* And in the other "side", to map keys -> clients */
-        db_blocked_entry = dictAddRaw(c->db->blocking_keys,keys[j], &db_blocked_existing_entry);
+        db_blocked_entry = dictAddRaw(c->db->blocking_keys, keys[j], &db_blocked_existing_entry);
 
         /* In case key[j] did not have blocking clients yet, we need to create a new list */
         if (db_blocked_entry != NULL) {
@@ -410,8 +398,8 @@ void blockForKeys(client *c, int btype, robj **keys, int numkeys, mstime_t timeo
         } else {
             l = dictGetVal(db_blocked_existing_entry);
         }
-        listAddNodeTail(l,c);
-        dictSetVal(c->bstate.keys,client_blocked_entry,listLast(l));
+        listAddNodeTail(l, c);
+        dictSetVal(c->bstate.keys, client_blocked_entry, listLast(l));
 
         /* We need to add the key to blocking_keys_unblock_on_nokey, if the client
          * wants to be awakened if key is deleted (like XREADGROUP) */
@@ -429,9 +417,8 @@ void blockForKeys(client *c, int btype, robj **keys, int numkeys, mstime_t timeo
     /* Currently we assume key blocking will require reprocessing the command.
      * However in case of modules, they have a different way to handle the reprocessing
      * which does not require setting the pending command flag */
-    if (btype != BLOCKED_MODULE)
-        c->flags |= CLIENT_PENDING_COMMAND;
-    blockClient(c,btype);
+    if (btype != BLOCKED_MODULE) c->flag.pending_command = 1;
+    blockClient(c, btype);
 }
 
 /* Helper function to unblock a client that's waiting in a blocking operation such as BLPOP.
@@ -440,12 +427,11 @@ static void unblockClientWaitingData(client *c) {
     dictEntry *de;
     dictIterator *di;
 
-    if (dictSize(c->bstate.keys) == 0)
-        return;
+    if (dictSize(c->bstate.keys) == 0) return;
 
     di = dictGetIterator(c->bstate.keys);
     /* The client may wait for multiple keys, so unblock it for every key. */
-    while((de = dictNext(di)) != NULL) {
+    while ((de = dictNext(di)) != NULL) {
         releaseBlockedEntry(c, de, 0);
     }
     dictReleaseIterator(di);
@@ -454,11 +440,11 @@ static void unblockClientWaitingData(client *c) {
 
 static blocking_type getBlockedTypeByType(int type) {
     switch (type) {
-        case OBJ_LIST: return BLOCKED_LIST;
-        case OBJ_ZSET: return BLOCKED_ZSET;
-        case OBJ_MODULE: return BLOCKED_MODULE;
-        case OBJ_STREAM: return BLOCKED_STREAM;
-        default: return BLOCKED_NONE;
+    case OBJ_LIST: return BLOCKED_LIST;
+    case OBJ_ZSET: return BLOCKED_ZSET;
+    case OBJ_MODULE: return BLOCKED_MODULE;
+    case OBJ_STREAM: return BLOCKED_STREAM;
+    default: return BLOCKED_NONE;
     }
 }
 
@@ -469,7 +455,7 @@ static blocking_type getBlockedTypeByType(int type) {
  * made by a script or in the context of MULTI/EXEC.
  *
  * The list will be finally processed by handleClientsBlockedOnKeys() */
-static void signalKeyAsReadyLogic(redisDb *db, robj *key, int type, int deleted) {
+static void signalKeyAsReadyLogic(serverDb *db, robj *key, int type, int deleted) {
     readyList *rl;
 
     /* Quick returns. */
@@ -478,8 +464,7 @@ static void signalKeyAsReadyLogic(redisDb *db, robj *key, int type, int deleted)
         /* The type can never block. */
         return;
     }
-    if (!server.blocked_clients_by_type[btype] &&
-        !server.blocked_clients_by_type[BLOCKED_MODULE]) {
+    if (!server.blocked_clients_by_type[btype] && !server.blocked_clients_by_type[BLOCKED_MODULE]) {
         /* No clients block on this type. Note: Blocked modules are represented
          * by BLOCKED_MODULE, even if the intention is to wake up by normal
          * types (list, zset, stream), so we need to check that there are no
@@ -489,13 +474,11 @@ static void signalKeyAsReadyLogic(redisDb *db, robj *key, int type, int deleted)
 
     if (deleted) {
         /* Key deleted and no clients blocking for this key? No need to queue it. */
-        if (dictFind(db->blocking_keys_unblock_on_nokey,key) == NULL)
-            return;
+        if (dictFind(db->blocking_keys_unblock_on_nokey, key) == NULL) return;
         /* Note: if we made it here it means the key is also present in db->blocking_keys */
     } else {
         /* No clients blocking for this key? No need to queue it. */
-        if (dictFind(db->blocking_keys,key) == NULL)
-            return;
+        if (dictFind(db->blocking_keys, key) == NULL) return;
     }
 
     dictEntry *de, *existing;
@@ -515,7 +498,7 @@ static void signalKeyAsReadyLogic(redisDb *db, robj *key, int type, int deleted)
     rl->key = key;
     rl->db = db;
     incrRefCount(key);
-    listAddNodeTail(server.ready_keys,rl);
+    listAddNodeTail(server.ready_keys, rl);
 }
 
 /* Helper function to wrap the logic of removing a client blocked key entry
@@ -539,8 +522,8 @@ static void releaseBlockedEntry(client *c, dictEntry *de, int remove_key) {
     pos = dictGetVal(de);
     /* Remove this client from the list of clients waiting for this key. */
     l = dictFetchValue(c->db->blocking_keys, key);
-    serverAssertWithInfo(c,key,l != NULL);
-    listUnlinkNode(l,pos);
+    serverAssertWithInfo(c, key, l != NULL);
+    listUnlinkNode(l, pos);
     /* If the list is empty we need to remove it to avoid wasting memory
      * We will also remove the key (if exists) from the blocking_keys_unblock_on_nokey dict.
      * However, in case the list is not empty, we will have to still perform reference accounting
@@ -550,25 +533,24 @@ static void releaseBlockedEntry(client *c, dictEntry *de, int remove_key) {
      * signalDeletedKeyAsReady. */
     if (listLength(l) == 0) {
         dictDelete(c->db->blocking_keys, key);
-        dictDelete(c->db->blocking_keys_unblock_on_nokey,key);
+        dictDelete(c->db->blocking_keys_unblock_on_nokey, key);
     } else if (c->bstate.unblock_on_nokey) {
-        unblock_on_nokey_entry = dictFind(c->db->blocking_keys_unblock_on_nokey,key);
+        unblock_on_nokey_entry = dictFind(c->db->blocking_keys_unblock_on_nokey, key);
         /* it is not possible to have a client blocked on nokey with no matching entry */
-        serverAssertWithInfo(c,key,unblock_on_nokey_entry != NULL);
+        serverAssertWithInfo(c, key, unblock_on_nokey_entry != NULL);
         if (!dictIncrUnsignedIntegerVal(unblock_on_nokey_entry, -1)) {
             /* in case the count is zero, we can delete the entry */
-             dictDelete(c->db->blocking_keys_unblock_on_nokey,key);
+            dictDelete(c->db->blocking_keys_unblock_on_nokey, key);
         }
     }
-    if (remove_key)
-        dictDelete(c->bstate.keys, key);
+    if (remove_key) dictDelete(c->bstate.keys, key);
 }
 
-void signalKeyAsReady(redisDb *db, robj *key, int type) {
+void signalKeyAsReady(serverDb *db, robj *key, int type) {
     signalKeyAsReadyLogic(db, key, type, 0);
 }
 
-void signalDeletedKeyAsReady(redisDb *db, robj *key, int type) {
+void signalDeletedKeyAsReady(serverDb *db, robj *key, int type) {
     signalKeyAsReadyLogic(db, key, type, 1);
 }
 
@@ -576,16 +558,15 @@ void signalDeletedKeyAsReady(redisDb *db, robj *key, int type) {
  * whenever a key is ready. we iterate over all the clients blocked on this key
  * and try to re-execute the command (in case the key is still available). */
 static void handleClientsBlockedOnKey(readyList *rl) {
-
     /* We serve clients in the same order they blocked for
      * this key, from the first blocked to the last. */
-    dictEntry *de = dictFind(rl->db->blocking_keys,rl->key);
+    dictEntry *de = dictFind(rl->db->blocking_keys, rl->key);
 
     if (de) {
         list *clients = dictGetVal(de);
         listNode *ln;
         listIter li;
-        listRewind(clients,&li);
+        listRewind(clients, &li);
 
         /* Avoid processing more than the initial count so that we're not stuck
          * in an endless loop in case the reprocessing of the command blocks again. */
@@ -601,9 +582,7 @@ static void handleClientsBlockedOnKey(readyList *rl) {
              * 3. In case of XREADGROUP call we will want to unblock on any change in object type
              *    or in case the key was deleted, since the group is no longer valid. */
             if ((o != NULL && (receiver->bstate.btype == getBlockedTypeByType(o->type))) ||
-                (o != NULL && (receiver->bstate.btype == BLOCKED_MODULE)) ||
-                (receiver->bstate.unblock_on_nokey))
-            {
+                (o != NULL && (receiver->bstate.btype == BLOCKED_MODULE)) || (receiver->bstate.unblock_on_nokey)) {
                 if (receiver->bstate.btype != BLOCKED_MODULE)
                     unblockClientOnKey(receiver, rl->key);
                 else
@@ -613,23 +592,19 @@ static void handleClientsBlockedOnKey(readyList *rl) {
     }
 }
 
-/* block a client due to wait command */
-void blockForReplication(client *c, mstime_t timeout, long long offset, long numreplicas) {
-    c->bstate.timeout = timeout;
-    c->bstate.reploffset = offset;
-    c->bstate.numreplicas = numreplicas;
-    listAddNodeHead(server.clients_waiting_acks,c);
-    blockClient(c,BLOCKED_WAIT);
-}
-
-/* block a client due to waitaof command */
-void blockForAofFsync(client *c, mstime_t timeout, long long offset, int numlocal, long numreplicas) {
+/* block a client for replica acknowledgement */
+void blockClientForReplicaAck(client *c, mstime_t timeout, long long offset, long numreplicas, int numlocal) {
     c->bstate.timeout = timeout;
     c->bstate.reploffset = offset;
     c->bstate.numreplicas = numreplicas;
     c->bstate.numlocal = numlocal;
-    listAddNodeHead(server.clients_waiting_acks,c);
-    blockClient(c,BLOCKED_WAITAOF);
+    listAddNodeHead(server.clients_waiting_acks, c);
+    /* Note that we remember the linked list node where the client is stored,
+     * this way removing the client in unblockClientWaitingReplicas() will not
+     * require a linear scan, but just a constant time operation. */
+    serverAssert(c->bstate.client_waiting_acks_list_node == NULL);
+    c->bstate.client_waiting_acks_list_node = listFirst(server.clients_waiting_acks);
+    blockClient(c, BLOCKED_WAIT);
 }
 
 /* Postpone client from executing a command. For example the server might be busy
@@ -637,16 +612,16 @@ void blockForAofFsync(client *c, mstime_t timeout, long long offset, int numloca
  * when the it is ready to accept them. */
 void blockPostponeClient(client *c) {
     c->bstate.timeout = 0;
-    blockClient(c,BLOCKED_POSTPONE);
+    blockClient(c, BLOCKED_POSTPONE);
     listAddNodeTail(server.postponed_clients, c);
-    c->postponed_list_node = listLast(server.postponed_clients);
+    serverAssert(c->bstate.postponed_list_node == NULL);
+    c->bstate.postponed_list_node = listLast(server.postponed_clients);
     /* Mark this client to execute its command */
-    c->flags |= CLIENT_PENDING_COMMAND;
+    c->flag.pending_command = 1;
 }
 
 /* Block client due to shutdown command */
 void blockClientShutdown(client *c) {
-    c->bstate.timeout = 0;
     blockClient(c, BLOCKED_SHUTDOWN);
 }
 
@@ -662,18 +637,16 @@ static void unblockClientOnKey(client *c, robj *key) {
 
     /* Only in case of blocking API calls, we might be blocked on several keys.
        however we should force unblock the entire blocking keys */
-    serverAssert(c->bstate.btype == BLOCKED_STREAM ||
-                c->bstate.btype == BLOCKED_LIST   ||
-                c->bstate.btype == BLOCKED_ZSET);
+    serverAssert(c->bstate.btype == BLOCKED_STREAM || c->bstate.btype == BLOCKED_LIST ||
+                 c->bstate.btype == BLOCKED_ZSET);
 
     /* We need to unblock the client before calling processCommandAndResetClient
      * because it checks the CLIENT_BLOCKED flag */
     unblockClient(c, 0);
     /* In case this client was blocked on keys during command
      * we need to re process the command again */
-    if (c->flags & CLIENT_PENDING_COMMAND) {
-        c->flags &= ~CLIENT_PENDING_COMMAND;
-        c->flags |= CLIENT_REPROCESSING_COMMAND;
+    if (c->flag.pending_command) {
+        c->flag.pending_command = 0;
         /* We want the command processing and the unblock handler (see RM_Call 'K' option)
          * to run atomically, this is why we must enter the execution unit here before
          * running the command, and exit the execution unit after calling the unblock handler (if exists).
@@ -683,8 +656,8 @@ static void unblockClientOnKey(client *c, robj *key) {
         server.current_client = c;
         enterExecutionUnit(1, 0);
         processCommandAndResetClient(c);
-        if (!(c->flags & CLIENT_BLOCKED)) {
-            if (c->flags & CLIENT_MODULE) {
+        if (!c->flag.blocked) {
+            if (c->flag.module) {
                 moduleCallCommandUnblockedHandler(c);
             } else {
                 queueClientForReprocessing(c);
@@ -692,8 +665,6 @@ static void unblockClientOnKey(client *c, robj *key) {
         }
         exitExecutionUnit();
         afterCommand(c);
-        /* Clear the reprocessing command flag after the proc is executed. */
-        c->flags &= ~CLIENT_REPROCESSING_COMMAND;
         server.current_client = old_client;
     }
 }
@@ -727,46 +698,30 @@ static void moduleUnblockClientOnKey(client *c, robj *key) {
  * we want to remove the pending flag to indicate we already responded to the
  * command with timeout reply. */
 void unblockClientOnTimeout(client *c) {
+    /* The client has been unlocked (in the moduleUnblocked list), return ASAP. */
+    if (c->bstate.btype == BLOCKED_MODULE && isModuleClientUnblocked(c)) return;
+
     replyToBlockedClientTimedOut(c);
-    if (c->flags & CLIENT_PENDING_COMMAND)
-        c->flags &= ~CLIENT_PENDING_COMMAND;
+    if (c->flag.pending_command) c->flag.pending_command = 0;
     unblockClient(c, 1);
 }
 
 /* Unblock a client which is currently Blocked with error.
  * If err_str is provided it will be used to reply to the blocked client */
 void unblockClientOnError(client *c, const char *err_str) {
-    if (err_str)
-        addReplyError(c, err_str);
+    if (err_str) addReplyError(c, err_str);
     updateStatsOnUnblock(c, 0, 0, 1);
-    if (c->flags & CLIENT_PENDING_COMMAND)
-        c->flags &= ~CLIENT_PENDING_COMMAND;
+    if (c->flag.pending_command) c->flag.pending_command = 0;
     unblockClient(c, 1);
 }
 
-/* sets blocking_keys to the total number of keys which has at least one client blocked on them
- * sets blocking_keys_on_nokey to the total number of keys which has at least one client
- * blocked on them to be written or deleted */
-void totalNumberOfBlockingKeys(unsigned long *blocking_keys, unsigned long *bloking_keys_on_nokey) {
-    unsigned long bkeys=0, bkeys_on_nokey=0;
-    for (int j = 0; j < server.dbnum; j++) {
-        bkeys += dictSize(server.db[j].blocking_keys);
-        bkeys_on_nokey += dictSize(server.db[j].blocking_keys_unblock_on_nokey);
-    }
-    if (blocking_keys)
-        *blocking_keys = bkeys;
-    if (bloking_keys_on_nokey)
-        *bloking_keys_on_nokey = bkeys_on_nokey;
-}
-
 void blockedBeforeSleep(void) {
     /* Handle precise timeouts of blocked clients. */
     handleBlockedClientsTimeout();
 
     /* Unblock all the clients blocked for synchronous replication
      * in WAIT or WAITAOF. */
-    if (listLength(server.clients_waiting_acks))
-        processClientsWaitingReplicas();
+    if (listLength(server.clients_waiting_acks)) processClientsWaitingReplicas();
 
     /* Try to process blocked clients every once in while.
      *
@@ -778,10 +733,8 @@ void blockedBeforeSleep(void) {
 
     /* Check if there are clients unblocked by modules that implement
      * blocking commands. */
-    if (moduleCount())
-        moduleHandleBlockedClients();
+    if (moduleCount()) moduleHandleBlockedClients();
 
     /* Try to process pending commands for clients that were just unblocked. */
-    if (listLength(server.unblocked_clients))
-        processUnblockedClients();
+    if (listLength(server.unblocked_clients)) processUnblockedClients();
 }
