@@ -88,7 +88,7 @@ static void luaEngineLoadHook(lua_State *lua, lua_Debug *ar) {
     if (load_ctx->timeout > 0 && duration > load_ctx->timeout) {
         lua_sethook(lua, luaEngineLoadHook, LUA_MASKLINE, 0);
 
-        luaPushError(lua,"FUNCTION LOAD timeout");
+        luaPushError(lua, "FUNCTION LOAD timeout");
         luaError(lua);
     }
 }
@@ -111,7 +111,7 @@ static int luaEngineCreate(void *engine_ctx, functionLibInfo *li, sds blob, size
     lua_getfield(lua, LUA_REGISTRYINDEX, LIBRARY_API_NAME);
     lua_setfield(lua, -2, "__index");
     lua_enablereadonlytable(lua, LUA_GLOBALSINDEX, 1); /* enable global protection */
-    lua_pop(lua, 1); /* pop the metatable */
+    lua_pop(lua, 1);                                   /* pop the metatable */
 
     /* compile the code */
     if (luaL_loadbuffer(lua, blob, sdslen(blob), "@user_function")) {
@@ -128,9 +128,9 @@ static int luaEngineCreate(void *engine_ctx, functionLibInfo *li, sds blob, size
     };
     luaSaveOnRegistry(lua, REGISTRY_LOAD_CTX_NAME, &load_ctx);
 
-    lua_sethook(lua,luaEngineLoadHook,LUA_MASKCOUNT,100000);
+    lua_sethook(lua, luaEngineLoadHook, LUA_MASKCOUNT, 100000);
     /* Run the compiled code to allow it to register functions */
-    if (lua_pcall(lua,0,0,0)) {
+    if (lua_pcall(lua, 0, 0, 0)) {
         errorInfo err_info = {0};
         luaExtractErrorInformation(lua, &err_info);
         *err = sdscatprintf(sdsempty(), "Error registering functions: %s", err_info.msg);
@@ -148,9 +148,9 @@ done:
     lua_getfield(lua, LUA_REGISTRYINDEX, GLOBALS_API_NAME);
     lua_setfield(lua, -2, "__index");
     lua_enablereadonlytable(lua, LUA_GLOBALSINDEX, 1); /* enable global protection */
-    lua_pop(lua, 1); /* pop the metatable */
+    lua_pop(lua, 1);                                   /* pop the metatable */
 
-    lua_sethook(lua,NULL,0,0); /* Disable hook */
+    lua_sethook(lua, NULL, 0, 0); /* Disable hook */
     luaSaveOnRegistry(lua, REGISTRY_LOAD_CTX_NAME, NULL);
     return ret;
 }
@@ -164,8 +164,7 @@ static void luaEngineCall(scriptRunCtx *run_ctx,
                           robj **keys,
                           size_t nkeys,
                           robj **args,
-                          size_t nargs)
-{
+                          size_t nargs) {
     luaEngineCtx *lua_engine_ctx = engine_ctx;
     lua_State *lua = lua_engine_ctx->lua;
     luaFunctionCtx *f_ctx = compiled_function;
@@ -205,11 +204,10 @@ static void luaEngineFreeFunction(void *engine_ctx, void *compiled_function) {
 }
 
 static void luaRegisterFunctionArgsInitialize(registerFunctionArgs *register_f_args,
-    sds name,
-    sds desc,
-    luaFunctionCtx *lua_f_ctx,
-    uint64_t flags)
-{
+                                              sds name,
+                                              sds desc,
+                                              luaFunctionCtx *lua_f_ctx,
+                                              uint64_t flags) {
     *register_f_args = (registerFunctionArgs){
         .name = name,
         .desc = desc,
@@ -232,22 +230,22 @@ static int luaRegisterFunctionReadFlags(lua_State *lua, uint64_t *flags) {
     int j = 1;
     int ret = C_ERR;
     int f_flags = 0;
-    while(1) {
-        lua_pushnumber(lua,j++);
-        lua_gettable(lua,-2);
-        int t = lua_type(lua,-1);
+    while (1) {
+        lua_pushnumber(lua, j++);
+        lua_gettable(lua, -2);
+        int t = lua_type(lua, -1);
         if (t == LUA_TNIL) {
-            lua_pop(lua,1);
+            lua_pop(lua, 1);
             break;
         }
         if (!lua_isstring(lua, -1)) {
-            lua_pop(lua,1);
+            lua_pop(lua, 1);
             goto done;
         }
 
         const char *flag_str = lua_tostring(lua, -1);
         int found = 0;
-        for (scriptFlag *flag = scripts_flags_def; flag->str ; ++flag) {
+        for (scriptFlag *flag = scripts_flags_def; flag->str; ++flag) {
             if (!strcasecmp(flag->str, flag_str)) {
                 f_flags |= flag->flag;
                 found = 1;
@@ -255,7 +253,7 @@ static int luaRegisterFunctionReadFlags(lua_State *lua, uint64_t *flags) {
             }
         }
         /* pops the value to continue the iteration */
-        lua_pop(lua,1);
+        lua_pop(lua, 1);
         if (!found) {
             /* flag not found */
             goto done;
@@ -276,7 +274,8 @@ static int luaRegisterFunctionReadNamedArgs(lua_State *lua, registerFunctionArgs
     luaFunctionCtx *lua_f_ctx = NULL;
     uint64_t flags = 0;
     if (!lua_istable(lua, 1)) {
-        err = "calling redis.register_function with a single argument is only applicable to Lua table (representing named arguments).";
+        err = "calling server.register_function with a single argument is only applicable to Lua table (representing "
+              "named arguments).";
         goto error;
     }
 
@@ -285,23 +284,23 @@ static int luaRegisterFunctionReadNamedArgs(lua_State *lua, registerFunctionArgs
     while (lua_next(lua, -2)) {
         /* Stack now: table, key, value */
         if (!lua_isstring(lua, -2)) {
-            err = "named argument key given to redis.register_function is not a string";
+            err = "named argument key given to server.register_function is not a string";
             goto error;
         }
         const char *key = lua_tostring(lua, -2);
         if (!strcasecmp(key, "function_name")) {
             if (!(name = luaGetStringSds(lua, -1))) {
-                err = "function_name argument given to redis.register_function must be a string";
+                err = "function_name argument given to server.register_function must be a string";
                 goto error;
             }
         } else if (!strcasecmp(key, "description")) {
             if (!(desc = luaGetStringSds(lua, -1))) {
-                err = "description argument given to redis.register_function must be a string";
+                err = "description argument given to server.register_function must be a string";
                 goto error;
             }
         } else if (!strcasecmp(key, "callback")) {
             if (!lua_isfunction(lua, -1)) {
-                err = "callback argument given to redis.register_function must be a function";
+                err = "callback argument given to server.register_function must be a function";
                 goto error;
             }
             int lua_function_ref = luaL_ref(lua, LUA_REGISTRYINDEX);
@@ -311,7 +310,7 @@ static int luaRegisterFunctionReadNamedArgs(lua_State *lua, registerFunctionArgs
             continue; /* value was already popped, so no need to pop it out. */
         } else if (!strcasecmp(key, "flags")) {
             if (!lua_istable(lua, -1)) {
-                err = "flags argument to redis.register_function must be a table representing function flags";
+                err = "flags argument to server.register_function must be a table representing function flags";
                 goto error;
             }
             if (luaRegisterFunctionReadFlags(lua, &flags) != C_OK) {
@@ -320,19 +319,19 @@ static int luaRegisterFunctionReadNamedArgs(lua_State *lua, registerFunctionArgs
             }
         } else {
             /* unknown argument was given, raise an error */
-            err = "unknown argument given to redis.register_function";
+            err = "unknown argument given to server.register_function";
             goto error;
         }
         lua_pop(lua, 1); /* pop the value to continue the iteration */
     }
 
     if (!name) {
-        err = "redis.register_function must get a function name argument";
+        err = "server.register_function must get a function name argument";
         goto error;
     }
 
     if (!lua_f_ctx) {
-        err = "redis.register_function must get a callback argument";
+        err = "server.register_function must get a callback argument";
         goto error;
     }
 
@@ -354,15 +353,14 @@ error:
 static int luaRegisterFunctionReadPositionalArgs(lua_State *lua, registerFunctionArgs *register_f_args) {
     char *err = NULL;
     sds name = NULL;
-    sds desc = NULL;
     luaFunctionCtx *lua_f_ctx = NULL;
     if (!(name = luaGetStringSds(lua, 1))) {
-        err = "first argument to redis.register_function must be a string";
+        err = "first argument to server.register_function must be a string";
         goto error;
     }
 
     if (!lua_isfunction(lua, 2)) {
-        err = "second argument to redis.register_function must be a function";
+        err = "second argument to server.register_function must be a function";
         goto error;
     }
 
@@ -377,7 +375,6 @@ static int luaRegisterFunctionReadPositionalArgs(lua_State *lua, registerFunctio
 
 error:
     if (name) sdsfree(name);
-    if (desc) sdsfree(desc);
     luaPushError(lua, err);
     return C_ERR;
 }
@@ -385,7 +382,7 @@ error:
 static int luaRegisterFunctionReadArgs(lua_State *lua, registerFunctionArgs *register_f_args) {
     int argc = lua_gettop(lua);
     if (argc < 1 || argc > 2) {
-        luaPushError(lua, "wrong number of arguments to redis.register_function");
+        luaPushError(lua, "wrong number of arguments to server.register_function");
         return C_ERR;
     }
 
@@ -401,7 +398,7 @@ static int luaRegisterFunction(lua_State *lua) {
 
     loadCtx *load_ctx = luaGetFromRegistry(lua, REGISTRY_LOAD_CTX_NAME);
     if (!load_ctx) {
-        luaPushError(lua, "redis.register_function can only be called on FUNCTION LOAD command");
+        luaPushError(lua, "server.register_function can only be called on FUNCTION LOAD command");
         return luaError(lua);
     }
 
@@ -410,7 +407,8 @@ static int luaRegisterFunction(lua_State *lua) {
     }
 
     sds err = NULL;
-    if (functionLibCreateFunction(register_f_args.name, register_f_args.lua_f_ctx, load_ctx->li, register_f_args.desc, register_f_args.f_flags, &err) != C_OK) {
+    if (functionLibCreateFunction(register_f_args.name, register_f_args.lua_f_ctx, load_ctx->li, register_f_args.desc,
+                                  register_f_args.f_flags, &err) != C_OK) {
         luaRegisterFunctionArgsDispose(lua, &register_f_args);
         luaPushError(lua, err);
         sdsfree(err);
@@ -425,7 +423,7 @@ int luaEngineInitEngine(void) {
     luaEngineCtx *lua_engine_ctx = zmalloc(sizeof(*lua_engine_ctx));
     lua_engine_ctx->lua = lua_open();
 
-    luaRegisterRedisAPI(lua_engine_ctx->lua);
+    luaRegisterServerAPI(lua_engine_ctx->lua);
 
     /* Register the library commands table and fields and store it to registry */
     lua_newtable(lua_engine_ctx->lua); /* load library globals */
@@ -452,25 +450,25 @@ int luaEngineInitEngine(void) {
 
     /* Save error handler to registry */
     lua_pushstring(lua_engine_ctx->lua, REGISTRY_ERROR_HANDLER_NAME);
-    char *errh_func =       "local dbg = debug\n"
-                            "debug = nil\n"
-                            "local error_handler = function (err)\n"
-                            "  local i = dbg.getinfo(2,'nSl')\n"
-                            "  if i and i.what == 'C' then\n"
-                            "    i = dbg.getinfo(3,'nSl')\n"
-                            "  end\n"
-                            "  if type(err) ~= 'table' then\n"
-                            "    err = {err='ERR ' .. tostring(err)}"
-                            "  end"
-                            "  if i then\n"
-                            "    err['source'] = i.source\n"
-                            "    err['line'] = i.currentline\n"
-                            "  end"
-                            "  return err\n"
-                            "end\n"
-                            "return error_handler";
+    char *errh_func = "local dbg = debug\n"
+                      "debug = nil\n"
+                      "local error_handler = function (err)\n"
+                      "  local i = dbg.getinfo(2,'nSl')\n"
+                      "  if i and i.what == 'C' then\n"
+                      "    i = dbg.getinfo(3,'nSl')\n"
+                      "  end\n"
+                      "  if type(err) ~= 'table' then\n"
+                      "    err = {err='ERR ' .. tostring(err)}"
+                      "  end"
+                      "  if i then\n"
+                      "    err['source'] = i.source\n"
+                      "    err['line'] = i.currentline\n"
+                      "  end"
+                      "  return err\n"
+                      "end\n"
+                      "return error_handler";
     luaL_loadbuffer(lua_engine_ctx->lua, errh_func, strlen(errh_func), "@err_handler_def");
-    lua_pcall(lua_engine_ctx->lua,0,1,0);
+    lua_pcall(lua_engine_ctx->lua, 0, 1, 0);
     lua_settable(lua_engine_ctx->lua, LUA_REGISTRYINDEX);
 
     lua_pushvalue(lua_engine_ctx->lua, LUA_GLOBALSINDEX);
@@ -496,11 +494,9 @@ int luaEngineInitEngine(void) {
     lua_enablereadonlytable(lua_engine_ctx->lua, -1, 1); /* protect the new global table */
     lua_replace(lua_engine_ctx->lua, LUA_GLOBALSINDEX);  /* set new global table as the new globals */
 
-    /* Set metatables of basic types (string, number, nil etc.) readonly. */
-    luaSetTableProtectionForBasicTypes(lua_engine_ctx->lua);
 
     engine *lua_engine = zmalloc(sizeof(*lua_engine));
-    *lua_engine = (engine) {
+    *lua_engine = (engine){
         .engine_ctx = lua_engine_ctx,
         .create = luaEngineCreate,
         .call = luaEngineCall,
