@@ -31,13 +31,10 @@
 #include "sds.h"
 #include "dict.h"
 #include "adlist.h"
-#include "atomicvar.h"
 
 #define LOAD_TIMEOUT_MS 500
 
-typedef enum {
-    restorePolicy_Flush, restorePolicy_Append, restorePolicy_Replace
-} restorePolicy;
+typedef enum { restorePolicy_Flush, restorePolicy_Append, restorePolicy_Replace } restorePolicy;
 
 static size_t engine_cache_memory = 0;
 
@@ -59,80 +56,70 @@ struct functionsLibCtx {
     dict *engines_stats; /* Per engine statistics */
 };
 
-typedef struct functionsLibMataData {
+typedef struct functionsLibMetaData {
     sds engine;
     sds name;
     sds code;
-} functionsLibMataData;
+} functionsLibMetaData;
 
 dictType engineDictType = {
-        dictSdsCaseHash,       /* hash function */
-        dictSdsDup,            /* key dup */
-        NULL,                  /* val dup */
-        dictSdsKeyCaseCompare, /* key compare */
-        dictSdsDestructor,     /* key destructor */
-        NULL,                  /* val destructor */
-        NULL                   /* allow to expand */
+    dictSdsCaseHash,       /* hash function */
+    dictSdsDup,            /* key dup */
+    dictSdsKeyCaseCompare, /* key compare */
+    dictSdsDestructor,     /* key destructor */
+    NULL,                  /* val destructor */
+    NULL                   /* allow to expand */
 };
 
 dictType functionDictType = {
-        dictSdsCaseHash,      /* hash function */
-        dictSdsDup,           /* key dup */
-        NULL,                 /* val dup */
-        dictSdsKeyCaseCompare,/* key compare */
-        dictSdsDestructor,    /* key destructor */
-        NULL,                 /* val destructor */
-        NULL                  /* allow to expand */
+    dictSdsCaseHash,       /* hash function */
+    dictSdsDup,            /* key dup */
+    dictSdsKeyCaseCompare, /* key compare */
+    dictSdsDestructor,     /* key destructor */
+    NULL,                  /* val destructor */
+    NULL                   /* allow to expand */
 };
 
 dictType engineStatsDictType = {
-        dictSdsCaseHash,      /* hash function */
-        dictSdsDup,           /* key dup */
-        NULL,                 /* val dup */
-        dictSdsKeyCaseCompare,/* key compare */
-        dictSdsDestructor,    /* key destructor */
-        engineStatsDispose,   /* val destructor */
-        NULL                  /* allow to expand */
+    dictSdsCaseHash,       /* hash function */
+    dictSdsDup,            /* key dup */
+    dictSdsKeyCaseCompare, /* key compare */
+    dictSdsDestructor,     /* key destructor */
+    engineStatsDispose,    /* val destructor */
+    NULL                   /* allow to expand */
 };
 
 dictType libraryFunctionDictType = {
-        dictSdsHash,          /* hash function */
-        dictSdsDup,           /* key dup */
-        NULL,                 /* val dup */
-        dictSdsKeyCompare,    /* key compare */
-        dictSdsDestructor,    /* key destructor */
-        engineFunctionDispose,/* val destructor */
-        NULL                  /* allow to expand */
+    dictSdsHash,           /* hash function */
+    dictSdsDup,            /* key dup */
+    dictSdsKeyCompare,     /* key compare */
+    dictSdsDestructor,     /* key destructor */
+    engineFunctionDispose, /* val destructor */
+    NULL                   /* allow to expand */
 };
 
 dictType librariesDictType = {
-        dictSdsHash,          /* hash function */
-        dictSdsDup,           /* key dup */
-        NULL,                 /* val dup */
-        dictSdsKeyCompare,    /* key compare */
-        dictSdsDestructor,    /* key destructor */
-        engineLibraryDispose, /* val destructor */
-        NULL                  /* allow to expand */
+    dictSdsHash,          /* hash function */
+    dictSdsDup,           /* key dup */
+    dictSdsKeyCompare,    /* key compare */
+    dictSdsDestructor,    /* key destructor */
+    engineLibraryDispose, /* val destructor */
+    NULL                  /* allow to expand */
 };
 
 /* Dictionary of engines */
 static dict *engines = NULL;
 
-/* Libraries Ctx.
- * Contains the dictionary that map a library name to library object,
- * Contains the dictionary that map a function name to function object,
- * and the cache memory used by all the functions */
+/* Libraries Ctx. */
 static functionsLibCtx *curr_functions_lib_ctx = NULL;
 
 static size_t functionMallocSize(functionInfo *fi) {
-    return zmalloc_size(fi) + sdsZmallocSize(fi->name)
-            + (fi->desc ? sdsZmallocSize(fi->desc) : 0)
-            + fi->li->ei->engine->get_function_memory_overhead(fi->function);
+    return zmalloc_size(fi) + sdsAllocSize(fi->name) + (fi->desc ? sdsAllocSize(fi->desc) : 0) +
+           fi->li->ei->engine->get_function_memory_overhead(fi->function);
 }
 
 static size_t libraryMallocSize(functionLibInfo *li) {
-    return zmalloc_size(li) + sdsZmallocSize(li->name)
-            + sdsZmallocSize(li->code);
+    return zmalloc_size(li) + sdsAllocSize(li->name) + sdsAllocSize(li->code);
 }
 
 static void engineStatsDispose(dict *d, void *obj) {
@@ -157,7 +144,7 @@ static void engineFunctionDispose(dict *d, void *obj) {
     zfree(fi);
 }
 
-static void engineLibraryFree(functionLibInfo* li) {
+static void engineLibraryFree(functionLibInfo *li) {
     if (!li) {
         return;
     }
@@ -214,12 +201,12 @@ void functionsLibCtxSwapWithCurrent(functionsLibCtx *new_lib_ctx) {
 }
 
 /* return the current functions ctx */
-functionsLibCtx* functionsLibCtxGetCurrent(void) {
+functionsLibCtx *functionsLibCtxGetCurrent(void) {
     return curr_functions_lib_ctx;
 }
 
 /* Create a new functions ctx */
-functionsLibCtx* functionsLibCtxCreate(void) {
+functionsLibCtx *functionsLibCtxCreate(void) {
     functionsLibCtx *ret = zmalloc(sizeof(functionsLibCtx));
     ret->libraries = dictCreate(&librariesDictType);
     ret->functions = dictCreate(&functionDictType);
@@ -247,7 +234,8 @@ functionsLibCtx* functionsLibCtxCreate(void) {
  */
 int functionLibCreateFunction(sds name, void *function, functionLibInfo *li, sds desc, uint64_t f_flags, sds *err) {
     if (functionsVerifyName(name) != C_OK) {
-        *err = sdsnew("Library names can only contain letters, numbers, or underscores(_) and must be at least one character long");
+        *err = sdsnew("Library names can only contain letters, numbers, or underscores(_) and must be at least one "
+                      "character long");
         return C_ERR;
     }
 
@@ -257,7 +245,7 @@ int functionLibCreateFunction(sds name, void *function, functionLibInfo *li, sds
     }
 
     functionInfo *fi = zmalloc(sizeof(*fi));
-    *fi = (functionInfo) {
+    *fi = (functionInfo){
         .name = name,
         .function = function,
         .li = li,
@@ -271,9 +259,9 @@ int functionLibCreateFunction(sds name, void *function, functionLibInfo *li, sds
     return C_OK;
 }
 
-static functionLibInfo* engineLibraryCreate(sds name, engineInfo *ei, sds code) {
+static functionLibInfo *engineLibraryCreate(sds name, engineInfo *ei, sds code) {
     functionLibInfo *li = zmalloc(sizeof(*li));
-    *li = (functionLibInfo) {
+    *li = (functionLibInfo){
         .name = sdsdup(name),
         .functions = dictCreate(&libraryFunctionDictType),
         .ei = ei,
@@ -282,7 +270,7 @@ static functionLibInfo* engineLibraryCreate(sds name, engineInfo *ei, sds code)
     return li;
 }
 
-static void libraryUnlink(functionsLibCtx *lib_ctx, functionLibInfo* li) {
+static void libraryUnlink(functionsLibCtx *lib_ctx, functionLibInfo *li) {
     dictIterator *iter = dictGetIterator(li->functions);
     dictEntry *entry = NULL;
     while ((entry = dictNext(iter))) {
@@ -304,7 +292,7 @@ static void libraryUnlink(functionsLibCtx *lib_ctx, functionLibInfo* li) {
     stats->n_functions -= dictSize(li->functions);
 }
 
-static void libraryLink(functionsLibCtx *lib_ctx, functionLibInfo* li) {
+static void libraryLink(functionsLibCtx *lib_ctx, functionLibInfo *li) {
     dictIterator *iter = dictGetIterator(li->functions);
     dictEntry *entry = NULL;
     while ((entry = dictNext(iter))) {
@@ -330,7 +318,8 @@ static void libraryLink(functionsLibCtx *lib_ctx, functionLibInfo* li) {
  * Return C_OK on success and C_ERR if aborted. If C_ERR is returned, set a relevant
  * error message on the 'err' out parameter.
  *  */
-static int libraryJoin(functionsLibCtx *functions_lib_ctx_dst, functionsLibCtx *functions_lib_ctx_src, int replace, sds *err) {
+static int
+libraryJoin(functionsLibCtx *functions_lib_ctx_dst, functionsLibCtx *functions_lib_ctx_src, int replace, sds *err) {
     int ret = C_ERR;
     dictIterator *iter = NULL;
     /* Stores the libraries we need to replace in case a revert is required.
@@ -349,7 +338,7 @@ static int libraryJoin(functionsLibCtx *functions_lib_ctx_dst, functionsLibCtx *
             } else {
                 if (!old_libraries_list) {
                     old_libraries_list = listCreate();
-                    listSetFreeMethod(old_libraries_list, (void (*)(void*))engineLibraryFree);
+                    listSetFreeMethod(old_libraries_list, (void (*)(void *))engineLibraryFree);
                 }
                 libraryUnlink(functions_lib_ctx_dst, old_li);
                 listAddNodeTail(old_libraries_list, old_li);
@@ -407,7 +396,7 @@ done:
 /* Register an engine, should be called once by the engine on startup and give the following:
  *
  * - engine_name - name of the engine to register
- * - engine_ctx - the engine ctx that should be used by Redis to interact with the engine */
+ * - engine_ctx - the engine ctx that should be used by the server to interact with the engine */
 int functionsRegisterEngine(const char *engine_name, engine *engine) {
     sds engine_name_sds = sdsnew(engine_name);
     if (dictFetchValue(engines, engine_name_sds)) {
@@ -417,15 +406,20 @@ int functionsRegisterEngine(const char *engine_name, engine *engine) {
     }
 
     client *c = createClient(NULL);
-    c->flags |= (CLIENT_DENY_BLOCKING | CLIENT_SCRIPT);
+    c->flag.deny_blocking = 1;
+    c->flag.script = 1;
+    c->flag.fake = 1;
     engineInfo *ei = zmalloc(sizeof(*ei));
-    *ei = (engineInfo ) { .name = engine_name_sds, .engine = engine, .c = c,};
+    *ei = (engineInfo){
+        .name = engine_name_sds,
+        .engine = engine,
+        .c = c,
+    };
 
     dictAdd(engines, engine_name_sds, ei);
 
-    engine_cache_memory += zmalloc_size(ei) + sdsZmallocSize(ei->name) +
-            zmalloc_size(engine) +
-            engine->get_engine_memory_overhead(engine->engine_ctx);
+    engine_cache_memory += zmalloc_size(ei) + sdsAllocSize(ei->name) + zmalloc_size(engine) +
+                           engine->get_engine_memory_overhead(engine->engine_ctx);
 
     return C_OK;
 }
@@ -451,7 +445,7 @@ void functionStatsCommand(client *c) {
         addReplyBulkCString(c, "command");
         client *script_client = scriptGetCaller();
         addReplyArrayLen(c, script_client->argc);
-        for (int i = 0 ; i < script_client->argc ; ++i) {
+        for (int i = 0; i < script_client->argc; ++i) {
             addReplyBulkCBuffer(c, script_client->argv[i]->ptr, sdslen(script_client->argv[i]->ptr));
         }
         addReplyBulkCString(c, "duration_ms");
@@ -478,7 +472,7 @@ void functionStatsCommand(client *c) {
 static void functionListReplyFlags(client *c, functionInfo *fi) {
     /* First count the number of flags we have */
     int flagcount = 0;
-    for (scriptFlag *flag = scripts_flags_def; flag->str ; ++flag) {
+    for (scriptFlag *flag = scripts_flags_def; flag->str; ++flag) {
         if (fi->f_flags & flag->flag) {
             ++flagcount;
         }
@@ -486,7 +480,7 @@ static void functionListReplyFlags(client *c, functionInfo *fi) {
 
     addReplySetLen(c, flagcount);
 
-    for (scriptFlag *flag = scripts_flags_def; flag->str ; ++flag) {
+    for (scriptFlag *flag = scripts_flags_def; flag->str; ++flag) {
         if (fi->f_flags & flag->flag) {
             addReplyStatus(c, flag->str);
         }
@@ -499,7 +493,6 @@ static void functionListReplyFlags(client *c, functionInfo *fi) {
  * Return general information about all the libraries:
  * * Library name
  * * The engine used to run the Library
- * * Library description
  * * Functions list
  * * Library code (if WITHCODE is given)
  *
@@ -510,7 +503,7 @@ static void functionListReplyFlags(client *c, functionInfo *fi) {
 void functionListCommand(client *c) {
     int with_code = 0;
     sds library_name = NULL;
-    for (int i = 2 ; i < c->argc ; ++i) {
+    for (int i = 2; i < c->argc; ++i) {
         robj *next_arg = c->argv[i];
         if (!with_code && !strcasecmp(next_arg->ptr, "withcode")) {
             with_code = 1;
@@ -545,7 +538,7 @@ void functionListCommand(client *c) {
             }
         }
         ++reply_len;
-        addReplyMapLen(c, with_code? 4 : 3);
+        addReplyMapLen(c, with_code ? 4 : 3);
         addReplyBulkCString(c, "library_name");
         addReplyBulkCBuffer(c, li->name, sdslen(li->name));
         addReplyBulkCString(c, "engine");
@@ -611,8 +604,7 @@ void functionKillCommand(client *c) {
 uint64_t fcallGetCommandFlags(client *c, uint64_t cmd_flags) {
     robj *function_name = c->argv[1];
     c->cur_script = dictFind(curr_functions_lib_ctx->functions, function_name->ptr);
-    if (!c->cur_script)
-        return cmd_flags;
+    if (!c->cur_script) return cmd_flags;
     functionInfo *fi = dictGetVal(c->cur_script);
     uint64_t script_flags = fi->f_flags;
     return scriptFlagsToCmdFlags(cmd_flags, script_flags);
@@ -620,12 +612,11 @@ uint64_t fcallGetCommandFlags(client *c, uint64_t cmd_flags) {
 
 static void fcallCommandGeneric(client *c, int ro) {
     /* Functions need to be fed to monitors before the commands they execute. */
-    replicationFeedMonitors(c,server.monitors,c->db->id,c->argv,c->argc);
+    replicationFeedMonitors(c, server.monitors, c->db->id, c->argv, c->argc);
 
     robj *function_name = c->argv[1];
     dictEntry *de = c->cur_script;
-    if (!de)
-        de = dictFind(curr_functions_lib_ctx->functions, function_name->ptr);
+    if (!de) de = dictFind(curr_functions_lib_ctx->functions, function_name->ptr);
     if (!de) {
         addReplyError(c, "Function not found");
         return;
@@ -649,11 +640,10 @@ static void fcallCommandGeneric(client *c, int ro) {
 
     scriptRunCtx run_ctx;
 
-    if (scriptPrepareForRun(&run_ctx, fi->li->ei->c, c, fi->name, fi->f_flags, ro) != C_OK)
-        return;
+    if (scriptPrepareForRun(&run_ctx, fi->li->ei->c, c, fi->name, fi->f_flags, ro) != C_OK) return;
 
-    engine->call(&run_ctx, engine->engine_ctx, fi->function, c->argv + 3, numkeys,
-                 c->argv + 3 + numkeys, c->argc - 3 - numkeys);
+    engine->call(&run_ctx, engine->engine_ctx, fi->function, c->argv + 3, numkeys, c->argv + 3 + numkeys,
+                 c->argc - 3 - numkeys);
     scriptResetRun(&run_ctx);
 }
 
@@ -681,7 +671,6 @@ void fcallroCommand(client *c) {
  * is saved separately with the following information:
  * * Library name
  * * Engine name
- * * Library description
  * * Library code
  * RDB_OPCODE_FUNCTION2 is saved before each library to present
  * that the payload is a library.
@@ -703,8 +692,7 @@ void functionDumpCommand(client *c) {
     payload.io.buffer.ptr = sdscatlen(payload.io.buffer.ptr, buf, 2);
 
     /* CRC64 */
-    crc = crc64(0, (unsigned char*) payload.io.buffer.ptr,
-                sdslen(payload.io.buffer.ptr));
+    crc = crc64(0, (unsigned char *)payload.io.buffer.ptr, sdslen(payload.io.buffer.ptr));
     memrev64ifbe(&crc);
     payload.io.buffer.ptr = sdscatlen(payload.io.buffer.ptr, &crc, 8);
 
@@ -748,7 +736,7 @@ void functionRestoreCommand(client *c) {
     }
 
     uint16_t rdbver;
-    if (verifyDumpPayload((unsigned char*)data, data_len, &rdbver) != C_OK) {
+    if (verifyDumpPayload((unsigned char *)data, data_len, &rdbver) != C_OK) {
         addReplyError(c, "DUMP payload version or checksum are wrong");
         return;
     }
@@ -783,7 +771,8 @@ void functionRestoreCommand(client *c) {
         functionsLibCtxSwapWithCurrent(functions_lib_ctx);
         functions_lib_ctx = NULL; /* avoid releasing the f_ctx in the end */
     } else {
-        if (libraryJoin(curr_functions_lib_ctx, functions_lib_ctx, restore_replicy == restorePolicy_Replace, &err) != C_OK) {
+        if (libraryJoin(curr_functions_lib_ctx, functions_lib_ctx, restore_replicy == restorePolicy_Replace, &err) !=
+            C_OK) {
             goto load_error;
         }
     }
@@ -810,14 +799,14 @@ void functionFlushCommand(client *c) {
         return;
     }
     int async = 0;
-    if (c->argc == 3 && !strcasecmp(c->argv[2]->ptr,"sync")) {
+    if (c->argc == 3 && !strcasecmp(c->argv[2]->ptr, "sync")) {
         async = 0;
-    } else if (c->argc == 3 && !strcasecmp(c->argv[2]->ptr,"async")) {
+    } else if (c->argc == 3 && !strcasecmp(c->argv[2]->ptr, "async")) {
         async = 1;
     } else if (c->argc == 2) {
         async = server.lazyfree_lazy_user_flush ? 1 : 0;
     } else {
-        addReplyError(c,"FUNCTION FLUSH only supports SYNC|ASYNC option");
+        addReplyError(c, "FUNCTION FLUSH only supports SYNC|ASYNC option");
         return;
     }
 
@@ -826,11 +815,12 @@ void functionFlushCommand(client *c) {
     /* Indicate that the command changed the data so it will be replicated and
      * counted as a data change (for persistence configuration) */
     server.dirty++;
-    addReply(c,shared.ok);
+    addReply(c, shared.ok);
 }
 
 /* FUNCTION HELP */
 void functionHelpCommand(client *c) {
+    /* clang-format off */
     const char *help[] = {
 "LOAD [REPLACE] <FUNCTION CODE>",
 "    Create a new library with the given library name and code.",
@@ -840,7 +830,6 @@ void functionHelpCommand(client *c) {
 "    Return general information on all the libraries:",
 "    * Library name",
 "    * The engine used to run the Library",
-"    * Library description",
 "    * Functions list",
 "    * Library code (if WITHCODE is given)",
 "    It also possible to get only function that matches a pattern using LIBRARYNAME argument.",
@@ -870,6 +859,7 @@ void functionHelpCommand(client *c) {
 "      libraries with the new libraries (notice that even on this option there is a chance of failure",
 "      in case of functions name collision with another library).",
 NULL };
+    /* clang-format on */
     addReplyHelp(c, help);
 }
 
@@ -878,13 +868,10 @@ static int functionsVerifyName(sds name) {
     if (sdslen(name) == 0) {
         return C_ERR;
     }
-    for (size_t i = 0 ; i < sdslen(name) ; ++i) {
+    for (size_t i = 0; i < sdslen(name); ++i) {
         char curr_char = name[i];
-        if ((curr_char >= 'a' && curr_char <= 'z') ||
-            (curr_char >= 'A' && curr_char <= 'Z') ||
-            (curr_char >= '0' && curr_char <= '9') ||
-            (curr_char == '_'))
-        {
+        if ((curr_char >= 'a' && curr_char <= 'z') || (curr_char >= 'A' && curr_char <= 'Z') ||
+            (curr_char >= '0' && curr_char <= '9') || (curr_char == '_')) {
             continue;
         }
         return C_ERR;
@@ -892,11 +879,9 @@ static int functionsVerifyName(sds name) {
     return C_OK;
 }
 
-int functionExtractLibMetaData(sds payload, functionsLibMataData *md, sds *err) {
+int functionExtractLibMetaData(sds payload, functionsLibMetaData *md, sds *err) {
     sds name = NULL;
-    sds desc = NULL;
     sds engine = NULL;
-    sds code = NULL;
     if (strncmp(payload, "#!", 2) != 0) {
         *err = sdsnew("Missing library metadata");
         return C_ERR;
@@ -918,7 +903,7 @@ int functionExtractLibMetaData(sds payload, functionsLibMataData *md, sds *err)
     }
     engine = sdsdup(parts[0]);
     sdsrange(engine, 2, -1);
-    for (int i = 1 ; i < numparts ; ++i) {
+    for (int i = 1; i < numparts; ++i) {
         sds part = parts[i];
         if (strncasecmp(part, "name=", 5) == 0) {
             if (name) {
@@ -948,14 +933,12 @@ int functionExtractLibMetaData(sds payload, functionsLibMataData *md, sds *err)
 
 error:
     if (name) sdsfree(name);
-    if (desc) sdsfree(desc);
     if (engine) sdsfree(engine);
-    if (code) sdsfree(code);
     sdsfreesplitres(parts, numparts);
     return C_ERR;
 }
 
-void functionFreeLibMetaData(functionsLibMataData *md) {
+void functionFreeLibMetaData(functionsLibMetaData *md) {
     if (md->code) sdsfree(md->code);
     if (md->name) sdsfree(md->name);
     if (md->engine) sdsfree(md->engine);
@@ -963,18 +946,19 @@ void functionFreeLibMetaData(functionsLibMataData *md) {
 
 /* Compile and save the given library, return the loaded library name on success
  * and NULL on failure. In case on failure the err out param is set with relevant error message */
-sds functionsCreateWithLibraryCtx(sds code, int replace, sds* err, functionsLibCtx *lib_ctx, size_t timeout) {
+sds functionsCreateWithLibraryCtx(sds code, int replace, sds *err, functionsLibCtx *lib_ctx, size_t timeout) {
     dictIterator *iter = NULL;
     dictEntry *entry = NULL;
     functionLibInfo *new_li = NULL;
     functionLibInfo *old_li = NULL;
-    functionsLibMataData md = {0};
+    functionsLibMetaData md = {0};
     if (functionExtractLibMetaData(code, &md, err) != C_OK) {
         return NULL;
     }
 
     if (functionsVerifyName(md.name)) {
-        *err = sdsnew("Library names can only contain letters, numbers, or underscores(_) and must be at least one character long");
+        *err = sdsnew("Library names can only contain letters, numbers, or underscores(_) and must be at least one "
+                      "character long");
         goto error;
     }
 
@@ -1053,7 +1037,7 @@ void functionLoadCommand(client *c) {
             replace = 1;
             continue;
         }
-        addReplyErrorFormat(c, "Unknown option given: %s", (char*)next_arg->ptr);
+        addReplyErrorFormat(c, "Unknown option given: %s", (char *)next_arg->ptr);
         return;
     }
 
@@ -1069,8 +1053,7 @@ void functionLoadCommand(client *c) {
     if (mustObeyClient(c)) {
         timeout = 0;
     }
-    if (!(library_name = functionsCreateWithLibraryCtx(code->ptr, replace, &err, curr_functions_lib_ctx, timeout)))
-    {
+    if (!(library_name = functionsCreateWithLibraryCtx(code->ptr, replace, &err, curr_functions_lib_ctx, timeout))) {
         addReplyErrorSds(c, err);
         return;
     }
@@ -1084,15 +1067,15 @@ void functionLoadCommand(client *c) {
 unsigned long functionsMemory(void) {
     dictIterator *iter = dictGetIterator(engines);
     dictEntry *entry = NULL;
-    size_t engines_nemory = 0;
+    size_t engines_memory = 0;
     while ((entry = dictNext(iter))) {
         engineInfo *ei = dictGetVal(entry);
         engine *engine = ei->engine;
-        engines_nemory += engine->get_used_memory(engine->engine_ctx);
+        engines_memory += engine->get_used_memory(engine->engine_ctx);
     }
     dictReleaseIterator(iter);
 
-    return engines_nemory;
+    return engines_memory;
 }
 
 /* Return memory overhead of all the engines combine */
@@ -1115,11 +1098,11 @@ unsigned long functionsLibNum(void) {
     return dictSize(curr_functions_lib_ctx->libraries);
 }
 
-dict* functionsLibGet(void) {
+dict *functionsLibGet(void) {
     return curr_functions_lib_ctx->libraries;
 }
 
-size_t functionsLibCtxfunctionsLen(functionsLibCtx *functions_ctx) {
+size_t functionsLibCtxFunctionsLen(functionsLibCtx *functions_ctx) {
     return dictSize(functions_ctx->functions);
 }
 
