@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009-2012, Salvatore Sanfilippo <antirez at gmail dot com>
+ * Copyright (c) 2009-2012, Redis Ltd.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -29,18 +29,19 @@
 
 #include "server.h"
 #include "cluster.h"
+#include "cluster_slot_stats.h"
 
 /* Structure to hold the pubsub related metadata. Currently used
  * for pubsub and pubsubshard feature. */
 typedef struct pubsubtype {
     int shard;
-    dict *(*clientPubSubChannels)(client*);
-    int (*subscriptionCount)(client*);
-    dict **serverPubSubChannels;
+    dict *(*clientPubSubChannels)(client *);
+    int (*subscriptionCount)(client *);
+    kvstore **serverPubSubChannels;
     robj **subscribeMsg;
     robj **unsubscribeMsg;
     robj **messageBulk;
-}pubsubtype;
+} pubsubtype;
 
 /*
  * Get client's global Pub/Sub channels subscription count.
@@ -55,19 +56,19 @@ int clientShardSubscriptionsCount(client *c);
 /*
  * Get client's global Pub/Sub channels dict.
  */
-dict* getClientPubSubChannels(client *c);
+dict *getClientPubSubChannels(client *c);
 
 /*
  * Get client's shard level Pub/Sub channels dict.
  */
-dict* getClientPubSubShardChannels(client *c);
+dict *getClientPubSubShardChannels(client *c);
 
 /*
  * Get list of channels client is subscribed to.
  * If a pattern is provided, the subset of channels is returned
  * matching the pattern.
  */
-void channelList(client *c, sds pat, dict* pubsub_channels);
+void channelList(client *c, sds pat, kvstore *pubsub_channels);
 
 /*
  * Pub/Sub type for global channels.
@@ -100,52 +101,52 @@ pubsubtype pubSubShardType = {
  *----------------------------------------------------------------------------*/
 
 /* Send a pubsub message of type "message" to the client.
- * Normally 'msg' is a Redis object containing the string to send as
+ * Normally 'msg' is an Object containing the string to send as
  * message. However if the caller sets 'msg' as NULL, it will be able
  * to send a special message (for instance an Array type) by using the
  * addReply*() API family. */
 void addReplyPubsubMessage(client *c, robj *channel, robj *msg, robj *message_bulk) {
-    uint64_t old_flags = c->flags;
-    c->flags |= CLIENT_PUSHING;
+    struct ClientFlags old_flags = c->flag;
+    c->flag.pushing = 1;
     if (c->resp == 2)
-        addReply(c,shared.mbulkhdr[3]);
+        addReply(c, shared.mbulkhdr[3]);
     else
-        addReplyPushLen(c,3);
-    addReply(c,message_bulk);
-    addReplyBulk(c,channel);
-    if (msg) addReplyBulk(c,msg);
-    if (!(old_flags & CLIENT_PUSHING)) c->flags &= ~CLIENT_PUSHING;
+        addReplyPushLen(c, 3);
+    addReply(c, message_bulk);
+    addReplyBulk(c, channel);
+    if (msg) addReplyBulk(c, msg);
+    if (!old_flags.pushing) c->flag.pushing = 0;
 }
 
 /* Send a pubsub message of type "pmessage" to the client. The difference
  * with the "message" type delivered by addReplyPubsubMessage() is that
  * this message format also includes the pattern that matched the message. */
 void addReplyPubsubPatMessage(client *c, robj *pat, robj *channel, robj *msg) {
-    uint64_t old_flags = c->flags;
-    c->flags |= CLIENT_PUSHING;
+    struct ClientFlags old_flags = c->flag;
+    c->flag.pushing = 1;
     if (c->resp == 2)
-        addReply(c,shared.mbulkhdr[4]);
+        addReply(c, shared.mbulkhdr[4]);
     else
-        addReplyPushLen(c,4);
-    addReply(c,shared.pmessagebulk);
-    addReplyBulk(c,pat);
-    addReplyBulk(c,channel);
-    addReplyBulk(c,msg);
-    if (!(old_flags & CLIENT_PUSHING)) c->flags &= ~CLIENT_PUSHING;
+        addReplyPushLen(c, 4);
+    addReply(c, shared.pmessagebulk);
+    addReplyBulk(c, pat);
+    addReplyBulk(c, channel);
+    addReplyBulk(c, msg);
+    if (!old_flags.pushing) c->flag.pushing = 0;
 }
 
 /* Send the pubsub subscription notification to the client. */
 void addReplyPubsubSubscribed(client *c, robj *channel, pubsubtype type) {
-    uint64_t old_flags = c->flags;
-    c->flags |= CLIENT_PUSHING;
+    struct ClientFlags old_flags = c->flag;
+    c->flag.pushing = 1;
     if (c->resp == 2)
-        addReply(c,shared.mbulkhdr[3]);
+        addReply(c, shared.mbulkhdr[3]);
     else
-        addReplyPushLen(c,3);
-    addReply(c,*type.subscribeMsg);
-    addReplyBulk(c,channel);
-    addReplyLongLong(c,type.subscriptionCount(c));
-    if (!(old_flags & CLIENT_PUSHING)) c->flags &= ~CLIENT_PUSHING;
+        addReplyPushLen(c, 3);
+    addReply(c, *type.subscribeMsg);
+    addReplyBulk(c, channel);
+    addReplyLongLong(c, type.subscriptionCount(c));
+    if (!old_flags.pushing) c->flag.pushing = 0;
 }
 
 /* Send the pubsub unsubscription notification to the client.
@@ -153,33 +154,33 @@ void addReplyPubsubSubscribed(client *c, robj *channel, pubsubtype type) {
  * unsubscribe command but there are no channels to unsubscribe from: we
  * still send a notification. */
 void addReplyPubsubUnsubscribed(client *c, robj *channel, pubsubtype type) {
-    uint64_t old_flags = c->flags;
-    c->flags |= CLIENT_PUSHING;
+    struct ClientFlags old_flags = c->flag;
+    c->flag.pushing = 1;
     if (c->resp == 2)
-        addReply(c,shared.mbulkhdr[3]);
+        addReply(c, shared.mbulkhdr[3]);
     else
-        addReplyPushLen(c,3);
+        addReplyPushLen(c, 3);
     addReply(c, *type.unsubscribeMsg);
     if (channel)
-        addReplyBulk(c,channel);
+        addReplyBulk(c, channel);
     else
         addReplyNull(c);
-    addReplyLongLong(c,type.subscriptionCount(c));
-    if (!(old_flags & CLIENT_PUSHING)) c->flags &= ~CLIENT_PUSHING;
+    addReplyLongLong(c, type.subscriptionCount(c));
+    if (!old_flags.pushing) c->flag.pushing = 0;
 }
 
 /* Send the pubsub pattern subscription notification to the client. */
 void addReplyPubsubPatSubscribed(client *c, robj *pattern) {
-    uint64_t old_flags = c->flags;
-    c->flags |= CLIENT_PUSHING;
+    struct ClientFlags old_flags = c->flag;
+    c->flag.pushing = 1;
     if (c->resp == 2)
-        addReply(c,shared.mbulkhdr[3]);
+        addReply(c, shared.mbulkhdr[3]);
     else
-        addReplyPushLen(c,3);
-    addReply(c,shared.psubscribebulk);
-    addReplyBulk(c,pattern);
-    addReplyLongLong(c,clientSubscriptionsCount(c));
-    if (!(old_flags & CLIENT_PUSHING)) c->flags &= ~CLIENT_PUSHING;
+        addReplyPushLen(c, 3);
+    addReply(c, shared.psubscribebulk);
+    addReplyBulk(c, pattern);
+    addReplyLongLong(c, clientSubscriptionsCount(c));
+    if (!old_flags.pushing) c->flag.pushing = 0;
 }
 
 /* Send the pubsub pattern unsubscription notification to the client.
@@ -187,19 +188,19 @@ void addReplyPubsubPatSubscribed(client *c, robj *pattern) {
  * punsubscribe command but there are no pattern to unsubscribe from: we
  * still send a notification. */
 void addReplyPubsubPatUnsubscribed(client *c, robj *pattern) {
-    uint64_t old_flags = c->flags;
-    c->flags |= CLIENT_PUSHING;
+    struct ClientFlags old_flags = c->flag;
+    c->flag.pushing = 1;
     if (c->resp == 2)
-        addReply(c,shared.mbulkhdr[3]);
+        addReply(c, shared.mbulkhdr[3]);
     else
-        addReplyPushLen(c,3);
-    addReply(c,shared.punsubscribebulk);
+        addReplyPushLen(c, 3);
+    addReply(c, shared.punsubscribebulk);
     if (pattern)
-        addReplyBulk(c,pattern);
+        addReplyBulk(c, pattern);
     else
         addReplyNull(c);
-    addReplyLongLong(c,clientSubscriptionsCount(c));
-    if (!(old_flags & CLIENT_PUSHING)) c->flags &= ~CLIENT_PUSHING;
+    addReplyLongLong(c, clientSubscriptionsCount(c));
+    if (!old_flags.pushing) c->flag.pushing = 0;
 }
 
 /*-----------------------------------------------------------------------------
@@ -208,15 +209,14 @@ void addReplyPubsubPatUnsubscribed(client *c, robj *pattern) {
 
 /* Return the number of pubsub channels + patterns is handled. */
 int serverPubsubSubscriptionCount(void) {
-    return dictSize(server.pubsub_channels) + dictSize(server.pubsub_patterns);
+    return kvstoreSize(server.pubsub_channels) + dictSize(server.pubsub_patterns);
 }
 
 /* Return the number of pubsub shard level channels is handled. */
 int serverPubsubShardSubscriptionCount(void) {
-    return dictSize(server.pubsubshard_channels);
+    return kvstoreSize(server.pubsubshard_channels);
 }
 
-
 /* Return the number of channels + patterns a client is subscribed to. */
 int clientSubscriptionsCount(client *c) {
     return dictSize(c->pubsub_channels) + dictSize(c->pubsub_patterns);
@@ -227,11 +227,11 @@ int clientShardSubscriptionsCount(client *c) {
     return dictSize(c->pubsubshard_channels);
 }
 
-dict* getClientPubSubChannels(client *c) {
+dict *getClientPubSubChannels(client *c) {
     return c->pubsub_channels;
 }
 
-dict* getClientPubSubShardChannels(client *c) {
+dict *getClientPubSubShardChannels(client *c) {
     return c->pubsubshard_channels;
 }
 
@@ -241,30 +241,54 @@ int clientTotalPubSubSubscriptionCount(client *c) {
     return clientSubscriptionsCount(c) + clientShardSubscriptionsCount(c);
 }
 
+void markClientAsPubSub(client *c) {
+    if (!c->flag.pubsub) {
+        c->flag.pubsub = 1;
+        server.pubsub_clients++;
+    }
+}
+
+void unmarkClientAsPubSub(client *c) {
+    if (c->flag.pubsub) {
+        c->flag.pubsub = 0;
+        server.pubsub_clients--;
+    }
+}
+
 /* Subscribe a client to a channel. Returns 1 if the operation succeeded, or
  * 0 if the client was already subscribed to that channel. */
 int pubsubSubscribeChannel(client *c, robj *channel, pubsubtype type) {
-    dictEntry *de;
-    list *clients = NULL;
+    dictEntry *de, *existing;
+    dict *clients = NULL;
     int retval = 0;
+    unsigned int slot = 0;
 
     /* Add the channel to the client -> channels hash table */
-    if (dictAdd(type.clientPubSubChannels(c),channel,NULL) == DICT_OK) {
+    void *position = dictFindPositionForInsert(type.clientPubSubChannels(c), channel, NULL);
+    if (position) { /* Not yet subscribed to this channel */
         retval = 1;
-        incrRefCount(channel);
         /* Add the client to the channel -> list of clients hash table */
-        de = dictFind(*type.serverPubSubChannels, channel);
-        if (de == NULL) {
-            clients = listCreate();
-            dictAdd(*type.serverPubSubChannels, channel, clients);
-            incrRefCount(channel);
-        } else {
-            clients = dictGetVal(de);
+        if (server.cluster_enabled && type.shard) {
+            slot = getKeySlot(channel->ptr);
         }
-        listAddNodeTail(clients,c);
+
+        de = kvstoreDictAddRaw(*type.serverPubSubChannels, slot, channel, &existing);
+
+        if (existing) {
+            clients = dictGetVal(existing);
+            channel = dictGetKey(existing);
+        } else {
+            clients = dictCreate(&clientDictType);
+            kvstoreDictSetVal(*type.serverPubSubChannels, slot, de, clients);
+            incrRefCount(channel);
+        }
+
+        serverAssert(dictAdd(clients, c, NULL) != DICT_ERR);
+        serverAssert(dictInsertAtPosition(type.clientPubSubChannels(c), channel, position));
+        incrRefCount(channel);
     }
     /* Notify the client */
-    addReplyPubsubSubscribed(c,channel,type);
+    addReplyPubsubSubscribed(c, channel, type);
     return retval;
 }
 
@@ -272,95 +296,90 @@ int pubsubSubscribeChannel(client *c, robj *channel, pubsubtype type) {
  * 0 if the client was not subscribed to the specified channel. */
 int pubsubUnsubscribeChannel(client *c, robj *channel, int notify, pubsubtype type) {
     dictEntry *de;
-    list *clients;
-    listNode *ln;
+    dict *clients;
     int retval = 0;
+    int slot = 0;
 
     /* Remove the channel from the client -> channels hash table */
     incrRefCount(channel); /* channel may be just a pointer to the same object
                             we have in the hash tables. Protect it... */
-    if (dictDelete(type.clientPubSubChannels(c),channel) == DICT_OK) {
+    if (dictDelete(type.clientPubSubChannels(c), channel) == DICT_OK) {
         retval = 1;
         /* Remove the client from the channel -> clients list hash table */
-        de = dictFind(*type.serverPubSubChannels, channel);
-        serverAssertWithInfo(c,NULL,de != NULL);
+        if (server.cluster_enabled && type.shard) {
+            slot = getKeySlot(channel->ptr);
+        }
+        de = kvstoreDictFind(*type.serverPubSubChannels, slot, channel);
+        serverAssertWithInfo(c, NULL, de != NULL);
         clients = dictGetVal(de);
-        ln = listSearchKey(clients,c);
-        serverAssertWithInfo(c,NULL,ln != NULL);
-        listDelNode(clients,ln);
-        if (listLength(clients) == 0) {
-            /* Free the list and associated hash entry at all if this was
+        serverAssertWithInfo(c, NULL, dictDelete(clients, c) == DICT_OK);
+        if (dictSize(clients) == 0) {
+            /* Free the dict and associated hash entry at all if this was
              * the latest client, so that it will be possible to abuse
-             * Redis PUBSUB creating millions of channels. */
-            dictDelete(*type.serverPubSubChannels, channel);
-            /* As this channel isn't subscribed by anyone, it's safe
-             * to remove the channel from the slot. */
-            if (server.cluster_enabled & type.shard) {
-                slotToChannelDel(channel->ptr);
-            }
+             * PUBSUB creating millions of channels. */
+            kvstoreDictDelete(*type.serverPubSubChannels, slot, channel);
         }
     }
     /* Notify the client */
     if (notify) {
-        addReplyPubsubUnsubscribed(c,channel,type);
+        addReplyPubsubUnsubscribed(c, channel, type);
     }
     decrRefCount(channel); /* it is finally safe to release it */
     return retval;
 }
 
-void pubsubShardUnsubscribeAllClients(robj *channel) {
-    int retval;
-    dictEntry *de = dictFind(server.pubsubshard_channels, channel);
-    serverAssertWithInfo(NULL,channel,de != NULL);
-    list *clients = dictGetVal(de);
-    if (listLength(clients) > 0) {
+/* Unsubscribe all shard channels in a slot. */
+void pubsubShardUnsubscribeAllChannelsInSlot(unsigned int slot) {
+    if (!kvstoreDictSize(server.pubsubshard_channels, slot)) return;
+
+    kvstoreDictIterator *kvs_di = kvstoreGetDictSafeIterator(server.pubsubshard_channels, slot);
+    dictEntry *de;
+    while ((de = kvstoreDictIteratorNext(kvs_di)) != NULL) {
+        robj *channel = dictGetKey(de);
+        dict *clients = dictGetVal(de);
         /* For each client subscribed to the channel, unsubscribe it. */
-        listIter li;
-        listNode *ln;
-        listRewind(clients, &li);
-        while ((ln = listNext(&li)) != NULL) {
-            client *c = listNodeValue(ln);
-            retval = dictDelete(c->pubsubshard_channels, channel);
-            serverAssertWithInfo(c,channel,retval == DICT_OK);
+        dictIterator *iter = dictGetIterator(clients);
+        dictEntry *entry;
+        while ((entry = dictNext(iter)) != NULL) {
+            client *c = dictGetKey(entry);
+            int retval = dictDelete(c->pubsubshard_channels, channel);
+            serverAssertWithInfo(c, channel, retval == DICT_OK);
             addReplyPubsubUnsubscribed(c, channel, pubSubShardType);
             /* If the client has no other pubsub subscription,
              * move out of pubsub mode. */
             if (clientTotalPubSubSubscriptionCount(c) == 0) {
-                c->flags &= ~CLIENT_PUBSUB;
+                unmarkClientAsPubSub(c);
             }
         }
+        dictReleaseIterator(iter);
+        kvstoreDictDelete(server.pubsubshard_channels, slot, channel);
     }
-    /* Delete the channel from server pubsubshard channels hash table. */
-    retval = dictDelete(server.pubsubshard_channels, channel);
-    /* Delete the channel from slots_to_channel mapping. */
-    slotToChannelDel(channel->ptr);
-    serverAssertWithInfo(NULL,channel,retval == DICT_OK);
-    decrRefCount(channel); /* it is finally safe to release it */
+    kvstoreReleaseDictIterator(kvs_di);
 }
 
-
-/* Subscribe a client to a pattern. Returns 1 if the operation succeeded, or 0 if the client was already subscribed to that pattern. */
+/* Subscribe a client to a pattern. Returns 1 if the operation succeeded, or 0 if the client was already subscribed to
+ * that pattern. */
 int pubsubSubscribePattern(client *c, robj *pattern) {
     dictEntry *de;
-    list *clients;
+    dict *clients;
     int retval = 0;
 
     if (dictAdd(c->pubsub_patterns, pattern, NULL) == DICT_OK) {
         retval = 1;
         incrRefCount(pattern);
         /* Add the client to the pattern -> list of clients hash table */
-        de = dictFind(server.pubsub_patterns,pattern);
+        de = dictFind(server.pubsub_patterns, pattern);
         if (de == NULL) {
-            clients = listCreate();
-            dictAdd(server.pubsub_patterns,pattern,clients);
+            clients = dictCreate(&clientDictType);
+            dictAdd(server.pubsub_patterns, pattern, clients);
             incrRefCount(pattern);
         } else {
             clients = dictGetVal(de);
         }
-        listAddNodeTail(clients,c);
+        serverAssert(dictAdd(clients, c, NULL) != DICT_ERR);
     }
     /* Notify the client */
-    addReplyPubsubPatSubscribed(c,pattern);
+    addReplyPubsubPatSubscribed(c, pattern);
     return retval;
 }
 
@@ -368,28 +387,25 @@ int pubsubSubscribePattern(client *c, robj *pattern) {
  * 0 if the client was not subscribed to the specified channel. */
 int pubsubUnsubscribePattern(client *c, robj *pattern, int notify) {
     dictEntry *de;
-    list *clients;
-    listNode *ln;
+    dict *clients;
     int retval = 0;
 
     incrRefCount(pattern); /* Protect the object. May be the same we remove */
     if (dictDelete(c->pubsub_patterns, pattern) == DICT_OK) {
         retval = 1;
         /* Remove the client from the pattern -> clients list hash table */
-        de = dictFind(server.pubsub_patterns,pattern);
-        serverAssertWithInfo(c,NULL,de != NULL);
+        de = dictFind(server.pubsub_patterns, pattern);
+        serverAssertWithInfo(c, NULL, de != NULL);
         clients = dictGetVal(de);
-        ln = listSearchKey(clients,c);
-        serverAssertWithInfo(c,NULL,ln != NULL);
-        listDelNode(clients,ln);
-        if (listLength(clients) == 0) {
-            /* Free the list and associated hash entry at all if this was
+        serverAssertWithInfo(c, NULL, dictDelete(clients, c) == DICT_OK);
+        if (dictSize(clients) == 0) {
+            /* Free the dict and associated hash entry at all if this was
              * the latest client. */
-            dictDelete(server.pubsub_patterns,pattern);
+            dictDelete(server.pubsub_patterns, pattern);
         }
     }
     /* Notify the client */
-    if (notify) addReplyPubsubPatUnsubscribed(c,pattern);
+    if (notify) addReplyPubsubPatUnsubscribed(c, pattern);
     decrRefCount(pattern);
     return retval;
 }
@@ -402,16 +418,16 @@ int pubsubUnsubscribeAllChannelsInternal(client *c, int notify, pubsubtype type)
         dictIterator *di = dictGetSafeIterator(type.clientPubSubChannels(c));
         dictEntry *de;
 
-        while((de = dictNext(di)) != NULL) {
+        while ((de = dictNext(di)) != NULL) {
             robj *channel = dictGetKey(de);
 
-            count += pubsubUnsubscribeChannel(c,channel,notify,type);
+            count += pubsubUnsubscribeChannel(c, channel, notify, type);
         }
         dictReleaseIterator(di);
     }
     /* We were subscribed to nothing? Still reply to the client. */
     if (notify && count == 0) {
-        addReplyPubsubUnsubscribed(c,NULL,type);
+        addReplyPubsubUnsubscribed(c, NULL, type);
     }
     return count;
 }
@@ -420,7 +436,7 @@ int pubsubUnsubscribeAllChannelsInternal(client *c, int notify, pubsubtype type)
  * Unsubscribe a client from all global channels.
  */
 int pubsubUnsubscribeAllChannels(client *c, int notify) {
-    int count = pubsubUnsubscribeAllChannelsInternal(c,notify,pubSubType);
+    int count = pubsubUnsubscribeAllChannelsInternal(c, notify, pubSubType);
     return count;
 }
 
@@ -432,17 +448,6 @@ int pubsubUnsubscribeShardAllChannels(client *c, int notify) {
     return count;
 }
 
-/*
- * Unsubscribe a client from provided shard subscribed channel(s).
- */
-void pubsubUnsubscribeShardChannels(robj **channels, unsigned int count) {
-    for (unsigned int j = 0; j < count; j++) {
-        /* Remove the channel from server and from the clients
-         * subscribed to it as well as notify them. */
-        pubsubShardUnsubscribeAllClients(channels[j]);
-    }
-}
-
 /* Unsubscribe from all the patterns. Return the number of patterns the
  * client was subscribed from. */
 int pubsubUnsubscribeAllPatterns(client *c, int notify) {
@@ -460,7 +465,7 @@ int pubsubUnsubscribeAllPatterns(client *c, int notify) {
     }
 
     /* We were subscribed to nothing? Still reply to the client. */
-    if (notify && count == 0) addReplyPubsubPatUnsubscribed(c,NULL);
+    if (notify && count == 0) addReplyPubsubPatUnsubscribed(c, NULL);
     return count;
 }
 
@@ -471,23 +476,25 @@ int pubsubPublishMessageInternal(robj *channel, robj *message, pubsubtype type)
     int receivers = 0;
     dictEntry *de;
     dictIterator *di;
-    listNode *ln;
-    listIter li;
+    int slot = -1;
 
     /* Send to clients listening for that channel */
-    de = dictFind(*type.serverPubSubChannels, channel);
+    if (server.cluster_enabled && type.shard) {
+        slot = keyHashSlot(channel->ptr, sdslen(channel->ptr));
+    }
+    de = kvstoreDictFind(*type.serverPubSubChannels, (slot == -1) ? 0 : slot, channel);
     if (de) {
-        list *list = dictGetVal(de);
-        listNode *ln;
-        listIter li;
-
-        listRewind(list,&li);
-        while ((ln = listNext(&li)) != NULL) {
-            client *c = ln->value;
-            addReplyPubsubMessage(c,channel,message,*type.messageBulk);
+        dict *clients = dictGetVal(de);
+        dictEntry *entry;
+        dictIterator *iter = dictGetIterator(clients);
+        while ((entry = dictNext(iter)) != NULL) {
+            client *c = dictGetKey(entry);
+            addReplyPubsubMessage(c, channel, message, *type.messageBulk);
+            clusterSlotStatsAddNetworkBytesOutForShardedPubSubInternalPropagation(c, slot);
             updateClientMemUsageAndBucket(c);
             receivers++;
         }
+        dictReleaseIterator(iter);
     }
 
     if (type.shard) {
@@ -499,21 +506,22 @@ int pubsubPublishMessageInternal(robj *channel, robj *message, pubsubtype type)
     di = dictGetIterator(server.pubsub_patterns);
     if (di) {
         channel = getDecodedObject(channel);
-        while((de = dictNext(di)) != NULL) {
+        while ((de = dictNext(di)) != NULL) {
             robj *pattern = dictGetKey(de);
-            list *clients = dictGetVal(de);
-            if (!stringmatchlen((char*)pattern->ptr,
-                                sdslen(pattern->ptr),
-                                (char*)channel->ptr,
-                                sdslen(channel->ptr),0)) continue;
+            dict *clients = dictGetVal(de);
+            if (!stringmatchlen((char *)pattern->ptr, sdslen(pattern->ptr), (char *)channel->ptr, sdslen(channel->ptr),
+                                0))
+                continue;
 
-            listRewind(clients,&li);
-            while ((ln = listNext(&li)) != NULL) {
-                client *c = listNodeValue(ln);
-                addReplyPubsubPatMessage(c,pattern,channel,message);
+            dictEntry *entry;
+            dictIterator *iter = dictGetIterator(clients);
+            while ((entry = dictNext(iter)) != NULL) {
+                client *c = dictGetKey(entry);
+                addReplyPubsubPatMessage(c, pattern, channel, message);
                 updateClientMemUsageAndBucket(c);
                 receivers++;
             }
+            dictReleaseIterator(iter);
         }
         decrRefCount(channel);
         dictReleaseIterator(di);
@@ -523,7 +531,7 @@ int pubsubPublishMessageInternal(robj *channel, robj *message, pubsubtype type)
 
 /* Publish a message to all the subscribers. */
 int pubsubPublishMessage(robj *channel, robj *message, int sharded) {
-    return pubsubPublishMessageInternal(channel, message, sharded? pubSubShardType : pubSubType);
+    return pubsubPublishMessageInternal(channel, message, sharded ? pubSubShardType : pubSubType);
 }
 
 /*-----------------------------------------------------------------------------
@@ -533,7 +541,7 @@ int pubsubPublishMessage(robj *channel, robj *message, int sharded) {
 /* SUBSCRIBE channel [channel ...] */
 void subscribeCommand(client *c) {
     int j;
-    if ((c->flags & CLIENT_DENY_BLOCKING) && !(c->flags & CLIENT_MULTI)) {
+    if (c->flag.deny_blocking && !c->flag.multi) {
         /**
          * A client that has CLIENT_DENY_BLOCKING flag on
          * expect a reply per command and so can not execute subscribe.
@@ -544,28 +552,28 @@ void subscribeCommand(client *c) {
         addReplyError(c, "SUBSCRIBE isn't allowed for a DENY BLOCKING client");
         return;
     }
-    for (j = 1; j < c->argc; j++)
-        pubsubSubscribeChannel(c,c->argv[j],pubSubType);
-    c->flags |= CLIENT_PUBSUB;
+    for (j = 1; j < c->argc; j++) pubsubSubscribeChannel(c, c->argv[j], pubSubType);
+    markClientAsPubSub(c);
 }
 
 /* UNSUBSCRIBE [channel ...] */
 void unsubscribeCommand(client *c) {
     if (c->argc == 1) {
-        pubsubUnsubscribeAllChannels(c,1);
+        pubsubUnsubscribeAllChannels(c, 1);
     } else {
         int j;
 
-        for (j = 1; j < c->argc; j++)
-            pubsubUnsubscribeChannel(c,c->argv[j],1,pubSubType);
+        for (j = 1; j < c->argc; j++) pubsubUnsubscribeChannel(c, c->argv[j], 1, pubSubType);
+    }
+    if (clientTotalPubSubSubscriptionCount(c) == 0) {
+        unmarkClientAsPubSub(c);
     }
-    if (clientTotalPubSubSubscriptionCount(c) == 0) c->flags &= ~CLIENT_PUBSUB;
 }
 
 /* PSUBSCRIBE pattern [pattern ...] */
 void psubscribeCommand(client *c) {
     int j;
-    if ((c->flags & CLIENT_DENY_BLOCKING) && !(c->flags & CLIENT_MULTI)) {
+    if (c->flag.deny_blocking && !c->flag.multi) {
         /**
          * A client that has CLIENT_DENY_BLOCKING flag on
          * expect a reply per command and so can not execute subscribe.
@@ -577,30 +585,29 @@ void psubscribeCommand(client *c) {
         return;
     }
 
-    for (j = 1; j < c->argc; j++)
-        pubsubSubscribePattern(c,c->argv[j]);
-    c->flags |= CLIENT_PUBSUB;
+    for (j = 1; j < c->argc; j++) pubsubSubscribePattern(c, c->argv[j]);
+    markClientAsPubSub(c);
 }
 
 /* PUNSUBSCRIBE [pattern [pattern ...]] */
 void punsubscribeCommand(client *c) {
     if (c->argc == 1) {
-        pubsubUnsubscribeAllPatterns(c,1);
+        pubsubUnsubscribeAllPatterns(c, 1);
     } else {
         int j;
 
-        for (j = 1; j < c->argc; j++)
-            pubsubUnsubscribePattern(c,c->argv[j],1);
+        for (j = 1; j < c->argc; j++) pubsubUnsubscribePattern(c, c->argv[j], 1);
+    }
+    if (clientTotalPubSubSubscriptionCount(c) == 0) {
+        unmarkClientAsPubSub(c);
     }
-    if (clientTotalPubSubSubscriptionCount(c) == 0) c->flags &= ~CLIENT_PUBSUB;
 }
 
 /* This function wraps pubsubPublishMessage and also propagates the message to cluster.
  * Used by the commands PUBLISH/SPUBLISH and their respective module APIs.*/
 int pubsubPublishMessageAndPropagateToCluster(robj *channel, robj *message, int sharded) {
     int receivers = pubsubPublishMessage(channel, message, sharded);
-    if (server.cluster_enabled)
-        clusterPropagatePublish(channel, message, sharded);
+    if (server.cluster_enabled) clusterPropagatePublish(channel, message, sharded);
     return receivers;
 }
 
@@ -611,105 +618,101 @@ void publishCommand(client *c) {
         return;
     }
 
-    int receivers = pubsubPublishMessageAndPropagateToCluster(c->argv[1],c->argv[2],0);
-    if (!server.cluster_enabled)
-        forceCommandPropagation(c,PROPAGATE_REPL);
-    addReplyLongLong(c,receivers);
+    int receivers = pubsubPublishMessageAndPropagateToCluster(c->argv[1], c->argv[2], 0);
+    if (!server.cluster_enabled) forceCommandPropagation(c, PROPAGATE_REPL);
+    addReplyLongLong(c, receivers);
 }
 
 /* PUBSUB command for Pub/Sub introspection. */
 void pubsubCommand(client *c) {
-    if (c->argc == 2 && !strcasecmp(c->argv[1]->ptr,"help")) {
+    if (c->argc == 2 && !strcasecmp(c->argv[1]->ptr, "help")) {
         const char *help[] = {
-"CHANNELS [<pattern>]",
-"    Return the currently active channels matching a <pattern> (default: '*').",
-"NUMPAT",
-"    Return number of subscriptions to patterns.",
-"NUMSUB [<channel> ...]",
-"    Return the number of subscribers for the specified channels, excluding",
-"    pattern subscriptions(default: no channels).",
-"SHARDCHANNELS [<pattern>]",
-"    Return the currently active shard level channels matching a <pattern> (default: '*').",
-"SHARDNUMSUB [<shardchannel> ...]",
-"    Return the number of subscribers for the specified shard level channel(s)",
-NULL
-        };
+            "CHANNELS [<pattern>]",
+            "    Return the currently active channels matching a <pattern> (default: '*').",
+            "NUMPAT",
+            "    Return number of subscriptions to patterns.",
+            "NUMSUB [<channel> ...]",
+            "    Return the number of subscribers for the specified channels, excluding",
+            "    pattern subscriptions(default: no channels).",
+            "SHARDCHANNELS [<pattern>]",
+            "    Return the currently active shard level channels matching a <pattern> (default: '*').",
+            "SHARDNUMSUB [<shardchannel> ...]",
+            "    Return the number of subscribers for the specified shard level channel(s)",
+            NULL};
         addReplyHelp(c, help);
-    } else if (!strcasecmp(c->argv[1]->ptr,"channels") &&
-        (c->argc == 2 || c->argc == 3))
-    {
+    } else if (!strcasecmp(c->argv[1]->ptr, "channels") && (c->argc == 2 || c->argc == 3)) {
         /* PUBSUB CHANNELS [<pattern>] */
         sds pat = (c->argc == 2) ? NULL : c->argv[2]->ptr;
         channelList(c, pat, server.pubsub_channels);
-    } else if (!strcasecmp(c->argv[1]->ptr,"numsub") && c->argc >= 2) {
+    } else if (!strcasecmp(c->argv[1]->ptr, "numsub") && c->argc >= 2) {
         /* PUBSUB NUMSUB [Channel_1 ... Channel_N] */
         int j;
 
-        addReplyArrayLen(c,(c->argc-2)*2);
+        addReplyArrayLen(c, (c->argc - 2) * 2);
         for (j = 2; j < c->argc; j++) {
-            list *l = dictFetchValue(server.pubsub_channels,c->argv[j]);
+            dict *d = kvstoreDictFetchValue(server.pubsub_channels, 0, c->argv[j]);
 
-            addReplyBulk(c,c->argv[j]);
-            addReplyLongLong(c,l ? listLength(l) : 0);
+            addReplyBulk(c, c->argv[j]);
+            addReplyLongLong(c, d ? dictSize(d) : 0);
         }
-    } else if (!strcasecmp(c->argv[1]->ptr,"numpat") && c->argc == 2) {
+    } else if (!strcasecmp(c->argv[1]->ptr, "numpat") && c->argc == 2) {
         /* PUBSUB NUMPAT */
-        addReplyLongLong(c,dictSize(server.pubsub_patterns));
-    } else if (!strcasecmp(c->argv[1]->ptr,"shardchannels") &&
-        (c->argc == 2 || c->argc == 3)) 
-    {
+        addReplyLongLong(c, dictSize(server.pubsub_patterns));
+    } else if (!strcasecmp(c->argv[1]->ptr, "shardchannels") && (c->argc == 2 || c->argc == 3)) {
         /* PUBSUB SHARDCHANNELS */
         sds pat = (c->argc == 2) ? NULL : c->argv[2]->ptr;
-        channelList(c,pat,server.pubsubshard_channels);
-    } else if (!strcasecmp(c->argv[1]->ptr,"shardnumsub") && c->argc >= 2) {
+        channelList(c, pat, server.pubsubshard_channels);
+    } else if (!strcasecmp(c->argv[1]->ptr, "shardnumsub") && c->argc >= 2) {
         /* PUBSUB SHARDNUMSUB [ShardChannel_1 ... ShardChannel_N] */
         int j;
-
-        addReplyArrayLen(c, (c->argc-2)*2);
+        addReplyArrayLen(c, (c->argc - 2) * 2);
         for (j = 2; j < c->argc; j++) {
-            list *l = dictFetchValue(server.pubsubshard_channels, c->argv[j]);
+            sds key = c->argv[j]->ptr;
+            unsigned int slot = server.cluster_enabled ? keyHashSlot(key, (int)sdslen(key)) : 0;
+            dict *clients = kvstoreDictFetchValue(server.pubsubshard_channels, slot, c->argv[j]);
 
-            addReplyBulk(c,c->argv[j]);
-            addReplyLongLong(c,l ? listLength(l) : 0);
+            addReplyBulk(c, c->argv[j]);
+            addReplyLongLong(c, clients ? dictSize(clients) : 0);
         }
     } else {
         addReplySubcommandSyntaxError(c);
     }
 }
 
-void channelList(client *c, sds pat, dict *pubsub_channels) {
-    dictIterator *di = dictGetIterator(pubsub_channels);
-    dictEntry *de;
+void channelList(client *c, sds pat, kvstore *pubsub_channels) {
     long mblen = 0;
     void *replylen;
+    unsigned int slot_cnt = kvstoreNumDicts(pubsub_channels);
 
     replylen = addReplyDeferredLen(c);
-    while((de = dictNext(di)) != NULL) {
-        robj *cobj = dictGetKey(de);
-        sds channel = cobj->ptr;
+    for (unsigned int i = 0; i < slot_cnt; i++) {
+        if (!kvstoreDictSize(pubsub_channels, i)) continue;
+        kvstoreDictIterator *kvs_di = kvstoreGetDictIterator(pubsub_channels, i);
+        dictEntry *de;
+        while ((de = kvstoreDictIteratorNext(kvs_di)) != NULL) {
+            robj *cobj = dictGetKey(de);
+            sds channel = cobj->ptr;
 
-        if (!pat || stringmatchlen(pat, sdslen(pat),
-                                   channel, sdslen(channel),0))
-        {
-            addReplyBulk(c,cobj);
-            mblen++;
+            if (!pat || stringmatchlen(pat, sdslen(pat), channel, sdslen(channel), 0)) {
+                addReplyBulk(c, cobj);
+                mblen++;
+            }
         }
+        kvstoreReleaseDictIterator(kvs_di);
     }
-    dictReleaseIterator(di);
-    setDeferredArrayLen(c,replylen,mblen);
+    setDeferredArrayLen(c, replylen, mblen);
 }
 
 /* SPUBLISH <shardchannel> <message> */
 void spublishCommand(client *c) {
-    int receivers = pubsubPublishMessageAndPropagateToCluster(c->argv[1],c->argv[2],1);
-    if (!server.cluster_enabled)
-        forceCommandPropagation(c,PROPAGATE_REPL);
-    addReplyLongLong(c,receivers);
+    int receivers = pubsubPublishMessageAndPropagateToCluster(c->argv[1], c->argv[2], 1);
+    if (!server.cluster_enabled) forceCommandPropagation(c, PROPAGATE_REPL);
+    addReplyLongLong(c, receivers);
 }
 
 /* SSUBSCRIBE shardchannel [shardchannel ...] */
 void ssubscribeCommand(client *c) {
-    if (c->flags & CLIENT_DENY_BLOCKING) {
+    if (c->flag.deny_blocking) {
         /* A client that has CLIENT_DENY_BLOCKING flag on
          * expect a reply per command and so can not execute subscribe. */
         addReplyError(c, "SSUBSCRIBE isn't allowed for a DENY BLOCKING client");
@@ -717,20 +720,11 @@ void ssubscribeCommand(client *c) {
     }
 
     for (int j = 1; j < c->argc; j++) {
-        /* A channel is only considered to be added, if a
-         * subscriber exists for it. And if a subscriber
-         * already exists the slotToChannel doesn't needs
-         * to be incremented. */
-        if (server.cluster_enabled &
-            (dictFind(*pubSubShardType.serverPubSubChannels, c->argv[j]) == NULL)) {
-            slotToChannelAdd(c->argv[j]->ptr);
-        }
         pubsubSubscribeChannel(c, c->argv[j], pubSubShardType);
     }
-    c->flags |= CLIENT_PUBSUB;
+    markClientAsPubSub(c);
 }
 
-
 /* SUNSUBSCRIBE [shardchannel [shardchannel ...]] */
 void sunsubscribeCommand(client *c) {
     if (c->argc == 1) {
@@ -740,7 +734,9 @@ void sunsubscribeCommand(client *c) {
             pubsubUnsubscribeChannel(c, c->argv[j], 1, pubSubShardType);
         }
     }
-    if (clientTotalPubSubSubscriptionCount(c) == 0) c->flags &= ~CLIENT_PUBSUB;
+    if (clientTotalPubSubSubscriptionCount(c) == 0) {
+        unmarkClientAsPubSub(c);
+    }
 }
 
 size_t pubsubMemOverhead(client *c) {
@@ -752,3 +748,8 @@ size_t pubsubMemOverhead(client *c) {
     mem += dictMemUsage(c->pubsubshard_channels);
     return mem;
 }
+
+int pubsubTotalSubscriptions(void) {
+    return dictSize(server.pubsub_patterns) + kvstoreSize(server.pubsub_channels) +
+           kvstoreSize(server.pubsubshard_channels);
+}
