@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, Salvatore Sanfilippo <antirez at gmail dot com>
+ * Copyright (c) 2016, Redis Ltd.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -42,13 +42,13 @@ int rdbCheckMode = 0;
 
 struct {
     rio *rio;
-    robj *key;                      /* Current key we are reading. */
-    int key_type;                   /* Current key type if != -1. */
-    unsigned long keys;             /* Number of keys processed. */
-    unsigned long expires;          /* Number of keys with an expire. */
-    unsigned long already_expired;  /* Number of keys already expired. */
-    int doing;                      /* The state while reading the RDB. */
-    int error_set;                  /* True if error is populated. */
+    robj *key;                     /* Current key we are reading. */
+    int key_type;                  /* Current key type if != -1. */
+    unsigned long keys;            /* Number of keys processed. */
+    unsigned long expires;         /* Number of keys with an expire. */
+    unsigned long already_expired; /* Number of keys already expired. */
+    int doing;                     /* The state while reading the RDB. */
+    int error_set;                 /* True if error is populated. */
     char error[1024];
 } rdbstate;
 
@@ -65,18 +65,8 @@ struct {
 #define RDB_CHECK_DOING_READ_MODULE_AUX 8
 #define RDB_CHECK_DOING_READ_FUNCTIONS 9
 
-char *rdb_check_doing_string[] = {
-    "start",
-    "read-type",
-    "read-expire",
-    "read-key",
-    "read-object-value",
-    "check-sum",
-    "read-len",
-    "read-aux",
-    "read-module-aux",
-    "read-functions"
-};
+char *rdb_check_doing_string[] = {"start",     "read-type", "read-expire", "read-key",        "read-object-value",
+                                  "check-sum", "read-len",  "read-aux",    "read-module-aux", "read-functions"};
 
 char *rdb_type_string[] = {
     "string",
@@ -121,20 +111,14 @@ void rdbCheckError(const char *fmt, ...) {
     va_end(ap);
 
     printf("--- RDB ERROR DETECTED ---\n");
-    printf("[offset %llu] %s\n",
-        (unsigned long long) (rdbstate.rio ?
-            rdbstate.rio->processed_bytes : 0), msg);
-    printf("[additional info] While doing: %s\n",
-        rdb_check_doing_string[rdbstate.doing]);
-    if (rdbstate.key)
-        printf("[additional info] Reading key '%s'\n",
-            (char*)rdbstate.key->ptr);
+    printf("[offset %llu] %s\n", (unsigned long long)(rdbstate.rio ? rdbstate.rio->processed_bytes : 0), msg);
+    printf("[additional info] While doing: %s\n", rdb_check_doing_string[rdbstate.doing]);
+    if (rdbstate.key) printf("[additional info] Reading key '%s'\n", (char *)rdbstate.key->ptr);
     if (rdbstate.key_type != -1)
-        printf("[additional info] Reading type %d (%s)\n",
-            rdbstate.key_type,
-            ((unsigned)rdbstate.key_type <
-             sizeof(rdb_type_string)/sizeof(char*)) ?
-                rdb_type_string[rdbstate.key_type] : "unknown");
+        printf("[additional info] Reading type %d (%s)\n", rdbstate.key_type,
+               ((unsigned)rdbstate.key_type < sizeof(rdb_type_string) / sizeof(char *))
+                   ? rdb_type_string[rdbstate.key_type]
+                   : "unknown");
     rdbShowGenericInfo();
 }
 
@@ -147,9 +131,7 @@ void rdbCheckInfo(const char *fmt, ...) {
     vsnprintf(msg, sizeof(msg), fmt, ap);
     va_end(ap);
 
-    printf("[offset %llu] %s\n",
-        (unsigned long long) (rdbstate.rio ?
-            rdbstate.rio->processed_bytes : 0), msg);
+    printf("[offset %llu] %s\n", (unsigned long long)(rdbstate.rio ? rdbstate.rio->processed_bytes : 0), msg);
 }
 
 /* Used inside rdb.c in order to log specific errors happening inside
@@ -202,29 +184,28 @@ int redis_check_rdb(char *rdbfilename, FILE *fp) {
     struct stat sb;
 
     int closefile = (fp == NULL);
-    if (fp == NULL && (fp = fopen(rdbfilename,"r")) == NULL) return 1;
+    if (fp == NULL && (fp = fopen(rdbfilename, "r")) == NULL) return 1;
 
-    if (fstat(fileno(fp), &sb) == -1)
-        sb.st_size = 0;
+    if (fstat(fileno(fp), &sb) == -1) sb.st_size = 0;
 
     startLoadingFile(sb.st_size, rdbfilename, RDBFLAGS_NONE);
-    rioInitWithFile(&rdb,fp);
+    rioInitWithFile(&rdb, fp);
     rdbstate.rio = &rdb;
     rdb.update_cksum = rdbLoadProgressCallback;
-    if (rioRead(&rdb,buf,9) == 0) goto eoferr;
+    if (rioRead(&rdb, buf, 9) == 0) goto eoferr;
     buf[9] = '\0';
-    if (memcmp(buf,"REDIS",5) != 0) {
+    if (memcmp(buf, "REDIS", 5) != 0) {
         rdbCheckError("Wrong signature trying to load DB from file");
         goto err;
     }
-    rdbver = atoi(buf+5);
+    rdbver = atoi(buf + 5);
     if (rdbver < 1 || rdbver > RDB_VERSION) {
-        rdbCheckError("Can't handle RDB format version %d",rdbver);
+        rdbCheckError("Can't handle RDB format version %d", rdbver);
         goto err;
     }
 
     expiretime = -1;
-    while(1) {
+    while (1) {
         robj *key, *val;
 
         /* Read type. */
@@ -251,11 +232,11 @@ int redis_check_rdb(char *rdbfilename, FILE *fp) {
         } else if (type == RDB_OPCODE_FREQ) {
             /* FREQ: LFU frequency. */
             uint8_t byte;
-            if (rioRead(&rdb,&byte,1) == 0) goto eoferr;
+            if (rioRead(&rdb, &byte, 1) == 0) goto eoferr;
             continue; /* Read next opcode. */
         } else if (type == RDB_OPCODE_IDLE) {
             /* IDLE: LRU idle time. */
-            if (rdbLoadLen(&rdb,NULL) == RDB_LENERR) goto eoferr;
+            if (rdbLoadLen(&rdb, NULL) == RDB_LENERR) goto eoferr;
             continue; /* Read next opcode. */
         } else if (type == RDB_OPCODE_EOF) {
             /* EOF: End of file, exit the main loop. */
@@ -263,8 +244,7 @@ int redis_check_rdb(char *rdbfilename, FILE *fp) {
         } else if (type == RDB_OPCODE_SELECTDB) {
             /* SELECTDB: Select the specified database. */
             rdbstate.doing = RDB_CHECK_DOING_READ_LEN;
-            if ((dbid = rdbLoadLen(&rdb,NULL)) == RDB_LENERR)
-                goto eoferr;
+            if ((dbid = rdbLoadLen(&rdb, NULL)) == RDB_LENERR) goto eoferr;
             rdbCheckInfo("Selecting DB ID %llu", (unsigned long long)dbid);
             selected_dbid = dbid;
             continue; /* Read type again. */
@@ -273,10 +253,8 @@ int redis_check_rdb(char *rdbfilename, FILE *fp) {
              * selected data base, in order to avoid useless rehashing. */
             uint64_t db_size, expires_size;
             rdbstate.doing = RDB_CHECK_DOING_READ_LEN;
-            if ((db_size = rdbLoadLen(&rdb,NULL)) == RDB_LENERR)
-                goto eoferr;
-            if ((expires_size = rdbLoadLen(&rdb,NULL)) == RDB_LENERR)
-                goto eoferr;
+            if ((db_size = rdbLoadLen(&rdb, NULL)) == RDB_LENERR) goto eoferr;
+            if ((expires_size = rdbLoadLen(&rdb, NULL)) == RDB_LENERR) goto eoferr;
             continue; /* Read type again. */
         } else if (type == RDB_OPCODE_AUX) {
             /* AUX: generic string-string fields. Use to add state to RDB
@@ -292,8 +270,7 @@ int redis_check_rdb(char *rdbfilename, FILE *fp) {
                 goto eoferr;
             }
 
-            rdbCheckInfo("AUX FIELD %s = '%s'",
-                (char*)auxkey->ptr, (char*)auxval->ptr);
+            rdbCheckInfo("AUX FIELD %s = '%s'", (char *)auxkey->ptr, (char *)auxval->ptr);
             decrRefCount(auxkey);
             decrRefCount(auxval);
             continue; /* Read type again. */
@@ -301,23 +278,23 @@ int redis_check_rdb(char *rdbfilename, FILE *fp) {
             /* AUX: Auxiliary data for modules. */
             uint64_t moduleid, when_opcode, when;
             rdbstate.doing = RDB_CHECK_DOING_READ_MODULE_AUX;
-            if ((moduleid = rdbLoadLen(&rdb,NULL)) == RDB_LENERR) goto eoferr;
-            if ((when_opcode = rdbLoadLen(&rdb,NULL)) == RDB_LENERR) goto eoferr;
-            if ((when = rdbLoadLen(&rdb,NULL)) == RDB_LENERR) goto eoferr;
+            if ((moduleid = rdbLoadLen(&rdb, NULL)) == RDB_LENERR) goto eoferr;
+            if ((when_opcode = rdbLoadLen(&rdb, NULL)) == RDB_LENERR) goto eoferr;
+            if ((when = rdbLoadLen(&rdb, NULL)) == RDB_LENERR) goto eoferr;
             if (when_opcode != RDB_MODULE_OPCODE_UINT) {
                 rdbCheckError("bad when_opcode");
                 goto err;
             }
 
             char name[10];
-            moduleTypeNameByID(name,moduleid);
+            moduleTypeNameByID(name, moduleid);
             rdbCheckInfo("MODULE AUX for: %s", name);
 
-            robj *o = rdbLoadCheckModuleValue(&rdb,name);
+            robj *o = rdbLoadCheckModuleValue(&rdb, name);
             decrRefCount(o);
             continue; /* Read type again. */
         } else if (type == RDB_OPCODE_FUNCTION_PRE_GA) {
-            rdbCheckError("Pre-release function format not supported %d",rdbver);
+            rdbCheckError("Pre-release function format not supported %d", rdbver);
             goto err;
         } else if (type == RDB_OPCODE_FUNCTION2) {
             sds err = NULL;
@@ -343,10 +320,9 @@ int redis_check_rdb(char *rdbfilename, FILE *fp) {
         rdbstate.keys++;
         /* Read value */
         rdbstate.doing = RDB_CHECK_DOING_READ_OBJECT_VALUE;
-        if ((val = rdbLoadObject(type,&rdb,key->ptr,selected_dbid,NULL)) == NULL) goto eoferr;
+        if ((val = rdbLoadObject(type, &rdb, key->ptr, selected_dbid, NULL)) == NULL) goto eoferr;
         /* Check if the key already expired. */
-        if (expiretime != -1 && expiretime < now)
-            rdbstate.already_expired++;
+        if (expiretime != -1 && expiretime < now) rdbstate.already_expired++;
         if (expiretime != -1) rdbstate.expires++;
         rdbstate.key = NULL;
         decrRefCount(key);
@@ -359,7 +335,7 @@ int redis_check_rdb(char *rdbfilename, FILE *fp) {
         uint64_t cksum, expected = rdb.cksum;
 
         rdbstate.doing = RDB_CHECK_DOING_CHECK_SUM;
-        if (rioRead(&rdb,&cksum,8) == 0) goto eoferr;
+        if (rioRead(&rdb, &cksum, 8) == 0) goto eoferr;
         memrev64ifbe(&cksum);
         if (cksum == 0) {
             rdbCheckInfo("RDB file was saved with checksum disabled: no check performed.");
@@ -387,22 +363,8 @@ err:
     return 1;
 }
 
-static sds checkRdbVersion(void) {
-    sds version;
-    version = sdscatprintf(sdsempty(), "%s", REDIS_VERSION);
-
-    /* Add git commit and working tree status when available */
-    if (strtoll(redisGitSHA1(),NULL,16)) {
-        version = sdscatprintf(version, " (git:%s", redisGitSHA1());
-        if (strtoll(redisGitDirty(),NULL,10))
-            version = sdscatprintf(version, "-dirty");
-        version = sdscat(version, ")");
-    }
-    return version;
-}
-
-/* RDB check main: called form server.c when Redis is executed with the
- * redis-check-rdb alias, on during RDB loading errors.
+/* RDB check main: called form server.c when the server is executed with the
+ * valkey-check-rdb alias, on during RDB loading errors.
  *
  * The function works in two ways: can be called with argc/argv as a
  * standalone executable, or called with a non NULL 'fp' argument if we
@@ -419,27 +381,26 @@ int redis_check_rdb_main(int argc, char **argv, FILE *fp) {
     if (argc != 2 && fp == NULL) {
         fprintf(stderr, "Usage: %s <rdb-file-name>\n", argv[0]);
         exit(1);
-    } else if (!strcmp(argv[1],"-v") || !strcmp(argv[1], "--version")) {
-        sds version = checkRdbVersion();
+    } else if (!strcmp(argv[1], "-v") || !strcmp(argv[1], "--version")) {
+        sds version = getVersion();
         printf("valkey-check-rdb %s\n", version);
         sdsfree(version);
         exit(0);
     }
 
     gettimeofday(&tv, NULL);
-    init_genrand64(((long long) tv.tv_sec * 1000000 + tv.tv_usec) ^ getpid());
+    init_genrand64(((long long)tv.tv_sec * 1000000 + tv.tv_usec) ^ getpid());
 
     /* In order to call the loading functions we need to create the shared
      * integer objects, however since this function may be called from
-     * an already initialized Redis instance, check if we really need to. */
-    if (shared.integers[0] == NULL)
-        createSharedObjects();
+     * an already initialized server instance, check if we really need to. */
+    if (shared.integers[0] == NULL) createSharedObjects();
     server.loading_process_events_interval_bytes = 0;
     server.sanitize_dump_payload = SANITIZE_DUMP_YES;
     rdbCheckMode = 1;
     rdbCheckInfo("Checking RDB file %s", argv[1]);
     rdbCheckSetupSignals();
-    int retval = redis_check_rdb(argv[1],fp);
+    int retval = redis_check_rdb(argv[1], fp);
     if (retval == 0) {
         rdbCheckInfo("\\o/ RDB looks OK! \\o/");
         rdbShowGenericInfo();
