@@ -51,7 +51,7 @@ start_server {tags {"repl network external:skip"}} {
 
         test {Slave is able to detect timeout during handshake} {
             wait_for_condition 50 1000 {
-                [log_file_matches $slave_log "*Timeout connecting to the MASTER*"]
+                [log_file_matches $slave_log "*Timeout connecting to the PRIMARY*"]
             } else {
                 fail "Replica is not able to detect timeout"
             }
@@ -108,7 +108,7 @@ start_server {tags {"repl external:skip"}} {
 
         test {BRPOPLPUSH replication, when blocking against empty list} {
             $A config resetstat
-            set rd [redis_deferring_client]
+            set rd [valkey_deferring_client]
             $rd brpoplpush a b 5
             r lpush a foo
             wait_for_condition 50 100 {
@@ -122,7 +122,7 @@ start_server {tags {"repl external:skip"}} {
 
         test {BRPOPLPUSH replication, list exists} {
             $A config resetstat
-            set rd [redis_deferring_client]
+            set rd [valkey_deferring_client]
             r lpush c 1
             r lpush c 2
             r lpush c 3
@@ -137,7 +137,7 @@ start_server {tags {"repl external:skip"}} {
             foreach whereto {left right} {
                 test "BLMOVE ($wherefrom, $whereto) replication, when blocking against empty list" {
                     $A config resetstat
-                    set rd [redis_deferring_client]
+                    set rd [valkey_deferring_client]
                     $rd blmove a b $wherefrom $whereto 5
                     r lpush a foo
                     wait_for_condition 50 100 {
@@ -151,7 +151,7 @@ start_server {tags {"repl external:skip"}} {
 
                 test "BLMOVE ($wherefrom, $whereto) replication, list exists" {
                     $A config resetstat
-                    set rd [redis_deferring_client]
+                    set rd [valkey_deferring_client]
                     r lpush c 1
                     r lpush c 2
                     r lpush c 3
@@ -165,8 +165,9 @@ start_server {tags {"repl external:skip"}} {
         }
 
         test {BLPOP followed by role change, issue #2473} {
-            set rd [redis_deferring_client]
+            set rd [valkey_deferring_client]
             $rd blpop foo 0 ; # Block while B is a master
+            wait_for_blocked_clients_count 1
 
             # Turn B into master of A
             $A slaveof no one
@@ -300,7 +301,7 @@ start_server {tags {"repl external:skip"}} {
     }
 }
 
-foreach mdl {no yes} {
+foreach mdl {no yes} dualchannel {no yes} {
     foreach sdl {disabled swapdb} {
         start_server {tags {"repl external:skip"} overrides {save {}}} {
             set master [srv 0 client]
@@ -316,7 +317,7 @@ foreach mdl {no yes} {
                     lappend slaves [srv 0 client]
                     start_server {overrides {save {}}} {
                         lappend slaves [srv 0 client]
-                        test "Connect multiple replicas at the same time (issue #141), master diskless=$mdl, replica diskless=$sdl" {
+                        test "Connect multiple replicas at the same time (issue #141), master diskless=$mdl, replica diskless=$sdl dual-channel-replication-enabled=$dualchannel" {
                             # start load handles only inside the test, so that the test can be skipped
                             set load_handle0 [start_bg_complex_data $master_host $master_port 9 100000000]
                             set load_handle1 [start_bg_complex_data $master_host $master_port 11 100000000]
@@ -325,7 +326,11 @@ foreach mdl {no yes} {
                             set load_handle4 [start_write_load $master_host $master_port 4]
                             after 5000 ;# wait for some data to accumulate so that we have RDB part for the fork
 
+                            $master config set dual-channel-replication-enabled $dualchannel
                             # Send SLAVEOF commands to slaves
+                            [lindex $slaves 0] config set dual-channel-replication-enabled $dualchannel
+                            [lindex $slaves 1] config set dual-channel-replication-enabled $dualchannel
+                            [lindex $slaves 2] config set dual-channel-replication-enabled $dualchannel
                             [lindex $slaves 0] config set repl-diskless-load $sdl
                             [lindex $slaves 1] config set repl-diskless-load $sdl
                             [lindex $slaves 2] config set repl-diskless-load $sdl
@@ -335,7 +340,7 @@ foreach mdl {no yes} {
 
                             # Wait for all the three slaves to reach the "online"
                             # state from the POV of the master.
-                            set retry 500
+                            set retry 1000
                             while {$retry} {
                                 set info [r -3 info]
                                 if {[string match {*slave0:*state=online*slave1:*state=online*slave2:*state=online*} $info]} {
@@ -374,6 +379,8 @@ foreach mdl {no yes} {
                             wait_for_ofs_sync $master [lindex $slaves 1]
                             wait_for_ofs_sync $master [lindex $slaves 2]
 
+                            assert [string match *replicas_waiting_psync:0* [$master info replication]]
+
                             # Check digests
                             set digest [$master debug digest]
                             set digest0 [[lindex $slaves 0] debug digest]
@@ -435,7 +442,7 @@ start_server {tags {"repl external:skip"} overrides {save {}}} {
 }
 
 # Diskless load swapdb when NOT async_loading (different master replid)
-foreach testType {Successful Aborted} {
+foreach testType {Successful Aborted} dualchannel {yes no} {
     start_server {tags {"repl external:skip"}} {
         set replica [srv 0 client]
         set replica_host [srv 0 host]
@@ -450,8 +457,10 @@ foreach testType {Successful Aborted} {
             $master config set repl-diskless-sync yes
             $master config set repl-diskless-sync-delay 0
             $master config set save ""
+            $master config set dual-channel-replication-enabled $dualchannel
             $replica config set repl-diskless-load swapdb
             $replica config set save ""
+            $replica config set dual-channel-replication-enabled $dualchannel
 
             # Put different data sets on the master and replica
             # We need to put large keys on the master since the replica replies to info only once in 2mb
@@ -471,7 +480,7 @@ foreach testType {Successful Aborted} {
                     # Start the replication process
                     $replica replicaof $master_host $master_port
 
-                    test {Diskless load swapdb (different replid): replica enter loading} {
+                    test "Diskless load swapdb (different replid): replica enter loading dual-channel-replication-enabled=$dualchannel" {
                         # Wait for the replica to start reading the rdb
                         wait_for_condition 100 100 {
                             [s -1 loading] eq 1
@@ -495,7 +504,7 @@ foreach testType {Successful Aborted} {
                         fail "Replica didn't disconnect"
                     }
 
-                    test {Diskless load swapdb (different replid): old database is exposed after replication fails} {
+                    test "Diskless load swapdb (different replid): old database is exposed after replication fails dual-channel=$dualchannel" {
                         # Ensure we see old values from replica
                         assert_equal [$replica get mykey] "myvalue"
 
@@ -517,7 +526,7 @@ foreach testType {Successful Aborted} {
                         fail "Master <-> Replica didn't finish sync"
                     }
 
-                    test {Diskless load swapdb (different replid): new database is exposed after swapping} {
+                    test "Diskless load swapdb (different replid): new database is exposed after swapping dual-channel=$dualchannel" {
                         # Ensure we don't see anymore the key that was stored only to replica and also that we don't get LOADING status
                         assert_equal [$replica GET mykey] ""
 
@@ -548,6 +557,7 @@ foreach testType {Successful Aborted} {
             $master config set save ""
             $replica config set repl-diskless-load swapdb
             $replica config set save ""
+            $replica config set dual-channel-replication-enabled no; # Doesn't work with swapdb
 
             # Set replica writable so we can check that a key we manually added is served
             # during replication and after failure, but disappears on success
@@ -574,12 +584,12 @@ foreach testType {Successful Aborted} {
 
             # Set a function value on replica to check status during loading, on failure and after swapping db
             $replica function load {#!lua name=test
-                redis.register_function('test', function() return 'hello1' end)
+                server.register_function('test', function() return 'hello1' end)
             }
 
             # Set a function value on master to check it reaches the replica when replication ends
             $master function load {#!lua name=test
-                redis.register_function('test', function() return 'hello2' end)
+                server.register_function('test', function() return 'hello2' end)
             }
 
             # Remember the sync_full stat before the client kill.
@@ -637,7 +647,7 @@ foreach testType {Successful Aborted} {
                     }
 
                     test {Busy script during async loading} {
-                        set rd_replica [redis_deferring_client -1]
+                        set rd_replica [valkey_deferring_client -1]
                         $replica config set lua-time-limit 10
                         $rd_replica eval {while true do end} 0
                         after 200
@@ -727,7 +737,7 @@ test {diskless loading short read} {
 
             # Set a function value to check short read handling on functions
             r function load {#!lua name=test
-                redis.register_function('test', function() return 'hello1' end)
+                server.register_function('test', function() return 'hello1' end)
             }
 
             for {set k 0} {$k < 3} {incr k} {
@@ -852,6 +862,7 @@ start_server {tags {"repl external:skip"} overrides {save ""}} {
     $master config set repl-diskless-sync yes
     $master config set repl-diskless-sync-delay 5
     $master config set repl-diskless-sync-max-replicas 2
+    $master config set dual-channel-replication-enabled "no"; # dual-channel-replication doesn't use pipe
     set master_host [srv 0 host]
     set master_port [srv 0 port]
     set master_pid [srv 0 pid]
@@ -1041,8 +1052,8 @@ test "diskless replication child being killed is collected" {
     }
 } {} {external:skip}
 
-foreach mdl {yes no} {
-    test "replication child dies when parent is killed - diskless: $mdl" {
+foreach mdl {yes no} dualchannel {yes no} {
+    test "replication child dies when parent is killed - diskless: $mdl dual-channel-replication-enabled: $dualchannel" {
         # when master is killed, make sure the fork child can detect that and exit
         start_server {tags {"repl"} overrides {save ""}} {
             set master [srv 0 client]
@@ -1056,6 +1067,7 @@ foreach mdl {yes no} {
             $master debug populate 10000
             start_server {overrides {save ""}} {
                 set replica [srv 0 client]
+                $replica config set dual-channel-replication-enabled $dualchannel
                 $replica replicaof $master_host $master_port
 
                 # wait for rdb child to start
@@ -1146,7 +1158,7 @@ test {replicaof right after disconnection} {
                     fail "Can't turn the instance into a replica"
                 }
 
-                set rd [redis_deferring_client -1]
+                set rd [valkey_deferring_client -1]
                 $rd debug sleep 1
                 after 100
 
@@ -1235,69 +1247,80 @@ test {Kill rdb child process if its dumping RDB is not useful} {
         }
     }
 } {} {external:skip}
-
-start_server {tags {"repl external:skip"}} {
-    set master1_host [srv 0 host]
-    set master1_port [srv 0 port]
-    r set a b
-
-    start_server {} {
-        set master2 [srv 0 client]
-        set master2_host [srv 0 host]
-        set master2_port [srv 0 port]
-        # Take 10s for dumping RDB
-        $master2 debug populate 10 master2 10
-        $master2 config set rdb-key-save-delay 1000000
+foreach dualchannel {yes no} {
+    start_server {tags {"repl external:skip"}} {
+        set master1 [srv 0 client]
+        set master1_host [srv 0 host]
+        set master1_port [srv 0 port]
+        $master1 config set dual-channel-replication-enabled $dualchannel
+        r set a b
 
         start_server {} {
-            set sub_replica [srv 0 client]
+            set master2 [srv 0 client]
+            set master2_host [srv 0 host]
+            set master2_port [srv 0 port]
+            # Take 10s for dumping RDB
+            $master2 debug populate 10 master2 10
+            $master2 config set rdb-key-save-delay 1000000
+            $master2 config set dual-channel-replication-enabled $dualchannel
 
             start_server {} {
-                # Full sync with master1
-                r slaveof $master1_host $master1_port
-                wait_for_sync r
-                assert_equal "b" [r get a]
+                set sub_replica [srv 0 client]
+                $sub_replica config set dual-channel-replication-enabled $dualchannel
 
-                # Let sub replicas sync with me
-                $sub_replica slaveof [srv 0 host] [srv 0 port]
-                wait_for_sync $sub_replica
-                assert_equal "b" [$sub_replica get a]
-
-                # Full sync with master2, and then kill master2 before finishing dumping RDB
-                r slaveof $master2_host $master2_port
-                wait_for_condition 50 100 {
-                    ([s -2 rdb_bgsave_in_progress] == 1) &&
-                    ([string match "*wait_bgsave*" [s -2 slave0]])
-                } else {
-                    fail "full sync didn't start"
-                }
-                catch {$master2 shutdown nosave}
-
-                test {Don't disconnect with replicas before loading transferred RDB when full sync} {
-                    assert ![log_file_matches [srv -1 stdout] "*Connection with master lost*"]
-                    # The replication id is not changed in entire replication chain
-                    assert_equal [s master_replid] [s -3 master_replid]
-                    assert_equal [s master_replid] [s -1 master_replid]
-                }
-
-                test {Discard cache master before loading transferred RDB when full sync} {
-                    set full_sync [s -3 sync_full]
-                    set partial_sync [s -3 sync_partial_ok]
-                    # Partial sync with master1
+                start_server {} {
+                    # Full sync with master1
+                    set replica [srv 0 client]
+                    $replica config set dual-channel-replication-enabled $dualchannel
                     r slaveof $master1_host $master1_port
                     wait_for_sync r
-                    # master1 accepts partial sync instead of full sync
-                    assert_equal $full_sync [s -3 sync_full]
-                    assert_equal [expr $partial_sync+1] [s -3 sync_partial_ok]
+                    assert_equal "b" [r get a]
 
-                    # Since master only partially sync replica, and repl id is not changed,
-                    # the replica doesn't disconnect with its sub-replicas
-                    assert_equal [s master_replid] [s -3 master_replid]
-                    assert_equal [s master_replid] [s -1 master_replid]
-                    assert ![log_file_matches [srv -1 stdout] "*Connection with master lost*"]
-                    # Sub replica just has one full sync, no partial resync.
-                    assert_equal 1 [s sync_full]
-                    assert_equal 0 [s sync_partial_ok]
+                    # Let sub replicas sync with me
+                    $sub_replica slaveof [srv 0 host] [srv 0 port]
+                    wait_for_sync $sub_replica
+                    assert_equal "b" [$sub_replica get a]
+
+                    # Full sync with master2, and then kill master2 before finishing dumping RDB
+                    r slaveof $master2_host $master2_port
+                    wait_for_condition 50 100 {
+                        ([s -2 rdb_bgsave_in_progress] == 1) &&
+                        ([string match "*wait_bgsave*" [s -2 slave0]])
+                    } else {
+                        fail "full sync didn't start"
+                    }
+                    catch {$master2 shutdown nosave}
+
+                    test "Don't disconnect with replicas before loading transferred RDB when full sync with dual-channel-replication $dualchannel" {
+                        assert ![log_file_matches [srv -1 stdout] "*Connection with master lost*"]
+                        # The replication id is not changed in entire replication chain
+                        assert_equal [s master_replid] [s -3 master_replid]
+                        assert_equal [s master_replid] [s -1 master_replid]
+                    }
+
+                    test "Discard cache master before loading transferred RDB when full sync with dual-channel-replication $dualchannel" {
+                        set full_sync [s -3 sync_full]
+                        set partial_sync [s -3 sync_partial_ok]
+                        # Partial sync with master1
+                        r slaveof $master1_host $master1_port
+                        wait_for_sync r
+                        # master1 accepts partial sync instead of full sync
+                        assert_equal $full_sync [s -3 sync_full]
+                        assert_equal [expr $partial_sync+1] [s -3 sync_partial_ok]
+
+                        # Since master only partially sync replica, and repl id is not changed,
+                        # the replica doesn't disconnect with its sub-replicas
+                        assert_equal [s master_replid] [s -3 master_replid]
+                        assert_equal [s master_replid] [s -1 master_replid]
+                        assert ![log_file_matches [srv -1 stdout] "*Connection with master lost*"]
+                        # Sub replica just has one full sync, no partial resync.
+                        assert_equal 1 [s sync_full]
+                        if {$dualchannel == "yes"} {
+                            assert_equal 1 [s sync_partial_ok]
+                        } else {
+                            assert_equal 0 [s sync_partial_ok]
+                        }
+                    }
                 }
             }
         }
@@ -1344,7 +1367,7 @@ test {replica can handle EINTR if use diskless load} {
 
 start_server {tags {"repl" "external:skip"}} {
     test "replica do not write the reply to the replication link - SYNC (_addReplyToBufferOrList)" {
-        set rd [redis_deferring_client]
+        set rd [valkey_deferring_client]
         set lines [count_log_lines 0]
 
         $rd sync
@@ -1361,7 +1384,7 @@ start_server {tags {"repl" "external:skip"}} {
     }
 
     test "replica do not write the reply to the replication link - SYNC (addReplyDeferredLen)" {
-        set rd [redis_deferring_client]
+        set rd [valkey_deferring_client]
         set lines [count_log_lines 0]
 
         $rd sync
@@ -1378,7 +1401,7 @@ start_server {tags {"repl" "external:skip"}} {
     }
 
     test "replica do not write the reply to the replication link - PSYNC (_addReplyToBufferOrList)" {
-        set rd [redis_deferring_client]
+        set rd [valkey_deferring_client]
         set lines [count_log_lines 0]
 
         $rd psync replicationid -1
@@ -1390,7 +1413,7 @@ start_server {tags {"repl" "external:skip"}} {
 
         # Check we got the warning logs about the GET command.
         verify_log_message 0 "*Replica generated a reply to command 'get', disconnecting it: *" $lines
-        verify_log_message 0 "*== CRITICAL == This master is sending an error to its replica: *" $lines
+        verify_log_message 0 "*== CRITICAL == This primary is sending an error to its replica: *" $lines
         verify_log_message 0 "*Replica can't interact with the keyspace*" $lines
 
         $rd close
@@ -1398,7 +1421,7 @@ start_server {tags {"repl" "external:skip"}} {
     }
 
     test "replica do not write the reply to the replication link - PSYNC (addReplyDeferredLen)" {
-        set rd [redis_deferring_client]
+        set rd [valkey_deferring_client]
         set lines [count_log_lines 0]
 
         $rd psync replicationid -1
@@ -1454,3 +1477,43 @@ start_server {tags {"repl external:skip"}} {
         }
     }
 }
+
+start_server {tags {"repl external:skip"}} {
+    set replica [srv 0 client]
+    $replica set replica_key replica_value
+
+    start_server {} {
+        set primary [srv 0 client]
+        set primary_host [srv 0 host]
+        set primary_port [srv 0 port]
+        $primary set primary_key primary_value
+
+        test {Replica keep the old data if RDB file save fails in disk-based replication} {
+            # Create a folder called 'dump.rdb' to trigger temp-rdb rename failure
+            # and it will cause RDB file save to fail at the rename.
+            set dump_rdb [file join [lindex [$replica config get dir] 1] dump.rdb]
+            if {[file exists $dump_rdb]} { exec rm -f $dump_rdb }
+            exec mkdir -p $dump_rdb
+
+            $replica replicaof $primary_host $primary_port
+
+            # Waiting for the rename to fail.
+            wait_for_log_messages -1 {"*Failed trying to rename the temp DB into dump.rdb*"} 0 1000 10
+
+            # Make sure the replica has not completed sync and keep the old data.
+            assert_equal {} [$replica get primary_key]
+            assert_equal {replica_value} [$replica get replica_key]
+
+            # Remove the test folder and make the rename success
+            exec rm -rf $dump_rdb
+            wait_for_condition 500 100 {
+                [$replica get primary_key] == {primary_value} &&
+                [$replica get replica_key] == {}
+            } else {
+                puts [$primary keys *]
+                puts [$replica keys *]
+                fail "Replication failed."
+            }
+        }
+    }
+}
