@@ -2,7 +2,7 @@
 start_server {tags {"tracking network logreqres:skip"}} {
     # Create a deferred client we'll use to redirect invalidation
     # messages to.
-    set rd_redirection [redis_deferring_client]
+    set rd_redirection [valkey_deferring_client]
     $rd_redirection client id
     set redir_id [$rd_redirection read]
     $rd_redirection subscribe __redis__:invalidate
@@ -10,11 +10,11 @@ start_server {tags {"tracking network logreqres:skip"}} {
 
     # Create another client that's not used as a redirection client
     # We should always keep this client's buffer clean
-    set rd [redis_deferring_client]
+    set rd [valkey_deferring_client]
 
     # Client to be used for SET and GET commands
     # We don't read this client's buffer
-    set rd_sg [redis_client] 
+    set rd_sg [valkey_client] 
 
     proc clean_all {} {
         uplevel {
@@ -24,8 +24,8 @@ start_server {tags {"tracking network logreqres:skip"}} {
             r CLIENT TRACKING off
             $rd QUIT
             $rd_redirection QUIT
-            set rd [redis_deferring_client]
-            set rd_redirection [redis_deferring_client]
+            set rd [valkey_deferring_client]
+            set rd_redirection [valkey_deferring_client]
             $rd_redirection client id
             set redir_id [$rd_redirection read]
             $rd_redirection subscribe __redis__:invalidate
@@ -269,7 +269,7 @@ start_server {tags {"tracking network logreqres:skip"}} {
         assert_equal PONG [r read]
 
         # Reinstantiating after QUIT
-        set rd_redirection [redis_deferring_client]
+        set rd_redirection [valkey_deferring_client]
         $rd_redirection CLIENT ID
         set redir_id [$rd_redirection read]
         $rd_redirection SUBSCRIBE __redis__:invalidate
@@ -396,7 +396,7 @@ start_server {tags {"tracking network logreqres:skip"}} {
     }
 
     test {BCAST with prefix collisions throw errors} {
-        set r [redis_client] 
+        set r [valkey_client] 
         catch {$r CLIENT TRACKING ON BCAST PREFIX FOOBAR PREFIX FOO} output
         assert_match {ERR Prefix 'FOOBAR'*'FOO'*} $output
 
@@ -745,7 +745,7 @@ start_server {tags {"tracking network logreqres:skip"}} {
 
     test {Regression test for #11715} {
         # This issue manifests when a client invalidates keys through the max key
-        # limit, which invalidates keys to get Redis below the limit, but no command is
+        # limit, which invalidates keys to get the server below the limit, but no command is
         # then executed. This can occur in several ways but the simplest is through 
         # multi-exec which queues commands.
         clean_all
@@ -814,7 +814,7 @@ start_server {tags {"tracking network logreqres:skip"}} {
     test {RESP3 based basic redirect invalidation with client reply off} {
         clean_all
 
-        set rd_redir [redis_deferring_client]
+        set rd_redir [valkey_deferring_client]
         $rd_redir hello 3
         $rd_redir read
 
@@ -876,11 +876,11 @@ start_server {tags {"tracking network logreqres:skip"}} {
     $rd close
 }
 
-# Just some extra covergae for --log-req-res, because we do not
+# Just some extra coverage for --log-req-res, because we do not
 # run the full tracking unit in that mode
 start_server {tags {"tracking network"}} {
     test {Coverage: Basic CLIENT CACHING} {
-        set rd_redirection [redis_deferring_client]
+        set rd_redirection [valkey_deferring_client]
         $rd_redirection client id
         set redir_id [$rd_redirection read]
         assert_equal {OK} [r CLIENT TRACKING on OPTIN REDIRECT $redir_id]
